import os
import shutil
import json
import requests
import polars as pl
from pnadcontinua.metadata import (
    SCHEMA, COLUMN_WIDTHS, VARIABLES_MAPPING, REGULAR_INCOME_GROUP
)
from pnadcontinua.constants import (
    DATA_FOLDER, TEMP_FOLDER, DATA_SOURCE_URL, DEFLATOR_SOURCE_URL
)
from pnadcontinua.utils import (
    download_zip_content_from_url, convert_fwf_to_parquet
)


CURRENT_DIR = os.path.dirname(__file__)
DATA_FOLDER_PATH = os.path.join(CURRENT_DIR, DATA_FOLDER)
TEMP_FOLDER_PATH = os.path.join(CURRENT_DIR, TEMP_FOLDER)


def get_quarter_data_urls():
    api_response = requests.get(DATA_SOURCE_URL).content
    api_response_json = json.loads(api_response)
    year_folders = [f for f in api_response_json if f["name"].isdigit()]
    quarter_file_urls = []
    for year_folder in year_folders:
        for quarter in year_folder["children"]:
            quarter_file_urls.append(quarter["url"])
    return quarter_file_urls


def download_data(file_url, variables):
    if variables:
        col_widths = {col: width for col, width in COLUMN_WIDTHS.items()
                      if col in variables}
        schema = {col: dtype for col, dtype in SCHEMA.items()
                  if col in variables}
    else:
        col_widths = {col: width for col, width in COLUMN_WIDTHS.items()}
        schema = {col: dtype for col, dtype in SCHEMA.items()}

    os.makedirs(DATA_FOLDER_PATH, exist_ok=True)
    if os.path.exists(TEMP_FOLDER_PATH):
        shutil.rmtree(TEMP_FOLDER_PATH)
    os.makedirs(TEMP_FOLDER_PATH)
    print(f"Downloading data...")
    download_zip_content_from_url(file_url, TEMP_FOLDER_PATH)
    file_name = os.listdir(TEMP_FOLDER_PATH)[0]
    fwf_file = os.path.join(TEMP_FOLDER_PATH, file_name)
    parquet_file = os.path.join(DATA_FOLDER_PATH, file_name[:-4] + ".parquet")
    parquet_file_temp = parquet_file + ".temp"
    convert_fwf_to_parquet(fwf_file, parquet_file_temp, col_widths, schema)
    os.replace(parquet_file_temp, parquet_file)
    os.remove(fwf_file)
    print("Download complete")


def load_data():
    return pl.scan_parquet(f"{DATA_FOLDER_PATH}/*.parquet")


def filter_cat_column(data, column, values):
    return data.filter(pl.col(column).is_in(values))


def filter_num_column(data, column, operation, value):
    if operation == "Igual a":
        return data.filter(pl.col(column) == value)
    elif operation == "Maior que":
        return data.filter(pl.col(column) > value)
    elif operation == "Menor que":
        return data.filter(pl.col(column) < value)
    elif operation == "Maior ou igual a":
        return data.filter(pl.col(column) >= value)
    elif operation == "Menor ou igual a":
        return data.filter(pl.col(column) <= value)


def count_total_expr():
    return (pl.col("V1028").sum().alias(f"qnt_pessoas"))


def ops_total_exprs(agg_list):
    agg_exprs = []
    for col, operation, deflator in agg_list:
        if deflator:
            col = col + "_def"
        if operation == "Soma":
            agg_exprs.append(
                ((pl.col(col) * pl.col("V1028")).sum()).alias(f"{col}_soma")
            )
        elif operation == "Média":
            agg_exprs.append(
                ((pl.col(col) * pl.col("V1028")).sum() / pl.col("V1028").sum())
                .alias(f"{col}_media")
            )
    return agg_exprs


def calculate_totals(data, group_cols, agg_list, include_count, deflator_data):
    if deflator_data is not None:
        data = data.join(
            deflator_data, on=["Ano", "Trimestre", "UF"], how="inner"
        )
        cols_to_def = set()
        for col, _, deflator in agg_list:
            if deflator:
                cols_to_def.add(col)
        for col in cols_to_def:
            def_type = "Habitual" if col in REGULAR_INCOME_GROUP else "Efetivo"
            data = data.with_columns(
                (pl.col(col) * pl.col(def_type)).alias(f"{col}_def")
            )
    agg_exprs = ops_total_exprs(agg_list)
    cnt_expr = None
    if include_count:
        cnt_expr = count_total_expr()
        agg_exprs.append(cnt_expr)
    result = data.group_by(group_cols).agg(agg_exprs).collect()
    for col in result.columns:
        if col in VARIABLES_MAPPING.keys():
            result = result.with_columns(
                pl.col(col)
                .cast(pl.Utf8)
                .replace(VARIABLES_MAPPING[col])
                .alias(col)
            )
    print(result)
    return result

def get_deflator():
    if os.path.exists(TEMP_FOLDER_PATH):
        shutil.rmtree(TEMP_FOLDER_PATH)
    os.makedirs(TEMP_FOLDER_PATH)
    download_zip_content_from_url(DEFLATOR_SOURCE_URL, TEMP_FOLDER_PATH)
    file_name = os.listdir(TEMP_FOLDER_PATH)[0]
    excel_file = os.path.join(TEMP_FOLDER_PATH, file_name)
    deflator = pl.read_excel(excel_file)
    os.remove(excel_file)
    quarters = {"01-02-03": 1, "04-05-06": 2, "07-08-09": 3, "10-11-12": 4}
    deflator = deflator.filter(pl.col("trim").is_in(quarters.keys()))
    deflator = deflator.with_columns(
        pl.col("trim").replace(quarters).alias("Trimestre").cast(pl.Int64),
        pl.col("Ano").cast(pl.Int64),
        pl.col("UF").cast(pl.Int64)
    )
    deflator = deflator.select(
        pl.col("Ano", "Trimestre", "UF", "Habitual", "Efetivo")
    )
    return deflator.lazy()



import os
import tkinter as tk
from pnadcontinua.gui.frames import (
    HomeFrame, MicrodataFrame, AggregationsFrame, TutorialFrame, VariablesFrame
)


class App(tk.Tk):
    def __init__(self):
        super().__init__()

        self.grid_rowconfigure(0, weight=1)
        self.grid_columnconfigure(0, weight=1)

        self.screen_width = self.winfo_screenwidth()
        self.screen_height = self.winfo_screenheight()

        self.window_size = 530

        self.title("PNAD Contínua")

        x_position = int((self.screen_width - self.window_size) / 2)
        y_position = int((self.screen_height - self.window_size) / 2)

        self.geometry(f"{self.window_size}x{self.window_size}+{x_position}+{y_position}")
        self.resizable(False, False)

        self.protocol("WM_DELETE_WINDOW", lambda: os._exit(0))

        # Creation of the frames
        self.home_frame = HomeFrame(self)
        self.microdata_frame = MicrodataFrame(self)
        self.aggregations_frame = AggregationsFrame(self)
        self.variables_frame = VariablesFrame(self)
        self.tutorial_frame = TutorialFrame(self)

        # Place the frames on the window
        self.home_frame.grid(row=0, column=0, sticky="nsew")
        self.microdata_frame.grid(row=0, column=0, sticky="nsew")
        self.aggregations_frame.grid(row=0, column=0, sticky="nsew")
        self.variables_frame.grid(row=0, column=0, sticky="nsew")
        self.tutorial_frame.grid(row=0, column=0, sticky="nsew")
        # Display the home frame
        self.show_frame(self.home_frame)

    def show_frame(self, frame):
        frame.tkraise()



DATA_FOLDER = ".data"
TEMP_FOLDER = ".temp"
DOWNLOADED_DATA_FILE = "downloaded.json"
DATA_SOURCE_URL = "https://servicodados.ibge.gov.br/api/v1/downloads/estatisticas?caminho=Trabalho_e_Rendimento/Pesquisa_Nacional_por_Amostra_de_Domicilios_continua/Trimestral/Microdados&nivel=1"
DEFLATOR_SOURCE_URL = "https://ftp.ibge.gov.br/Trabalho_e_Rendimento/Pesquisa_Nacional_por_Amostra_de_Domicilios_continua/Trimestral/Microdados/Documentacao/Deflatores.zip"
TUTORIAL_URL = "https://github.com/cleomarfelipe/pnadcontinua#readme"



CNAE_OCUPATION = {
    1111: "Legisladores",
    1112: "Dirigentes superiores da administração pública",
    1113: "Chefes de pequenas populações",
    1114: "Dirigentes de organizações que apresentam um interesse especial",
    1120: "Diretores gerais e gerentes gerais",
    1211: "Dirigentes financeiros",
    1212: "Dirigentes de recursos humanos",
    1213: "Dirigentes de políticas e planejamento",
    1219: "Dirigentes de administração e de serviços não classificados anteriormente",
    1221: "Dirigentes de vendas e  comercialização",
    1222: "Dirigentes de publicidade e relações públicas",
    1223: "Dirigentes de pesquisa e desenvolvimento",
    1311: "Dirigentes de produção agropecuária e silvicultura",
    1312: "Dirigentes de produção da aquicultura e pesca",
    1321: "Dirigentes de indústria de transformação",
    1322: "Dirigentes de explorações de mineração",
    1323: "Dirigentes de empresas de construção",
    1324: "Dirigentes de empresas de abastecimento, distribuição e afins",
    1330: "Dirigentes de serviços de tecnologia da informação e comunicações",
    1341: "Dirigentes de serviços de cuidados infantis",
    1342: "Dirigentes de serviços de saúde",
    1343: "Dirigentes de serviços de cuidado a pessoas idosas",
    1344: "Dirigentes de serviços de bem-estar social",
    1345: "Dirigentes de serviços de educação",
    1346: "Gerentes de sucursais de bancos, de serviços financeiros e de seguros",
    1349: "Dirigentes e gerentes de serviços profissionais não classificados anteriormente",
    1411: "Gerentes de hotéis",
    1412: "Gerentes de restaurantes",
    1420: "Gerentes de comércios atacadistas e varejistas",
    1431: "Gerentes de centros esportivos,  de diversão e culturais",
    1439: "Gerentes de serviços não classificados anteriormente",
    2111: "Físicos e astrônomos",
    2112: "Meteorologistas",
    2113: "Químicos",
    2114: "Geólogos e geofísicos",
    2120: "Matemáticos, atuários e estatísticos",
    2131: "Biólogos, botânicos, zoólogos e afins",
    2132: "Agrônomos e afins",
    2133: "Profissionais da proteção do meio ambiente",
    2141: "Engenheiros industriais e de produção",
    2142: "Engenheiros civis",
    2143: "Engenheiros de meio ambiente",
    2144: "Engenheiros mecânicos",
    2145: "Engenheiros químicos",
    2146: "Engenheiros de minas, metalúrgicos e afins",
    2149: "Engenheiros não classificados anteriormente",
    2151: "Engenheiros eletricistas",
    2152: "Engenheiros eletrônicos",
    2153: "Engenheiros em telecomunicações",
    2161: "Arquitetos de edificações",
    2162: "Arquitetos paisagistas",
    2163: "Desenhistas de produtos e  vestuário",
    2164: "Urbanistas e engenheiros de trânsito",
    2165: "Cartógrafos e agrimensores",
    2166: "Desenhistas gráficos e de multimídia",
    2211: "Médicos gerais",
    2212: "Médicos especialistas",
    2221: "Profissionais de enfermagem",
    2222: "Profissionais de partos",
    2230: "Profissionais da medicina tradicional e alternativa",
    2240: "Paramédicos",
    2250: "Veterinários",
    2261: "Dentistas",
    2262: "Farmacêuticos",
    2263: "Profissionais da saúde e da higiene laboral e ambiental",
    2264: "Fisioterapeutas",
    2265: "Dietistas e nutricionistas",
    2266: "Fonoaudiólogos e logopedistas",
    2267: "Optometristas",
    2269: "Profissionais da saúde não classificados anteriormente",
    2310: "Professores de universidades e do ensino superior",
    2320: "Professores de formação profissional",
    2330: "Professores do ensino médio",
    2341: "Professores do ensino fundamental",
    2342: "Professores do ensino pré-escolar",
    2351: "Especialistas em métodos pedagógicos",
    2352: "Educadores para necessidades especiais",
    2353: "Outros professores de idiomas",
    2354: "Outros professores de música",
    2355: "Outros professores de artes",
    2356: "Instrutores em tecnologias da informação",
    2359: "Profissionais de ensino não classificados anteriormente",
    2411: "Contadores",
    2412: "Assessores financeiros e em investimentos",
    2413: "Analistas financeiros",
    2421: "Analistas de gestão e administração",
    2422: "Especialistas em políticas de administração",
    2423: "Especialistas em políticas e serviços de pessoal e afins",
    2424: "Especialistas em formação de pessoal",
    2431: "Profissionais da publicidade e da comercialização",
    2432: "Profissionais de relações públicas",
    2433: "Profissionais de vendas técnicas e médicas (exclusive tic)",
    2434: "Profissionais de vendas de tecnologia da informação e comunicações",
    2511: "Analistas de sistemas",
    2512: "Desenvolvedores de programas e aplicativos (software)",
    2513: "Desenvolvedores de páginas de internet (web) e multimídia",
    2514: "Programadores de aplicações",
    2519: "Desenvolvedores e analistas de programas e aplicativos (software) e multimídia não classificados anteriormente",
    2521: "Desenhistas e administradores de bases de dados",
    2522: "Administradores de sistemas",
    2523: "Profissionais em rede de computadores",
    2529: "Especialistas em base de dados e em redes de computadores não classificados anteriormente",
    2611: "Advogados e juristas",
    2612: "Juízes",
    2619: "Profissionais em direito não classificados anteriormente",
    2621: "Arquivologistas e curadores de museus",
    2622: "Bibliotecários, documentaristas e afins",
    2631: "Economistas",
    2632: "Sociólogos, antropólogos e afins",
    2633: "Filósofos, historiadores e especialistas em ciência política",
    2634: "Psicólogos",
    2635: "Assistentes sociais",
    2636: "Ministros de cultos religiosos, missionários e afins",
    2641: "Escritores",
    2642: "Jornalistas",
    2643: "Tradutores, intérpretes e linguistas",
    2651: "Artistas plásticos",
    2652: "Músicos, cantores e compositores",
    2653: "Bailarinos e coreógrafos",
    2654: "Diretores de cinema, de teatro e afins",
    2655: "Atores",
    2656: "Locutores de rádio, televisão e outros meios de comunicação",
    2659: "Artistas criativos e interpretativos não classificados anteriormente",
    3111: "Técnicos em ciências físicas e químicas",
    3112: "Técnicos em engenharia civil",
    3113: "Eletrotécnicos",
    3114: "Técnicos em eletrônica",
    3115: "Técnicos em engenharia mecânica",
    3116: "Técnicos em química industrial",
    3117: "Técnicos em engenharia de minas e metalurgia",
    3118: "Desenhistas e projetistas técnicos",
    3119: "Técnicos em ciências físicas e da engenharia não classificados anteriormente",
    3121: "Supervisores da mineração",
    3122: "Supervisores de indústrias de transformação",
    3123: "Supervisores da construção",
    3131: "Operadores de instalações de produção de energia",
    3132: "Operadores de incineradores, instalações de tratamento de água e afins",
    3133: "Controladores de instalações de processamento de produtos químicos",
    3134: "Operadores de instalações de refino de petróleo e gás natural",
    3135: "Controladores de processos de produção de metais",
    3139: "Técnicos em controle de processos não classificados anteriormente",
    3141: "Técnicos e profissionais de nível médio em ciências biológicas (exclusive da medicina)",
    3142: "Técnicos agropecuários",
    3143: "Técnicos florestais",
    3151: "Oficiais maquinistas em navegação",
    3152: "Capitães, oficiais de coberta e práticos",
    3153: "Pilotos de aviação e afins",
    3154: "Controladores de tráfego aéreo",
    3155: "Técnicos em segurança aeronáutica",
    3211: "Técnicos em aparelhos de diagnóstico e tratamento médico",
    3212: "Técnicos de laboratórios médicos",
    3213: "Técnicos e assistentes farmacêuticos",
    3214: "Técnicos de próteses médicas e dentárias",
    3221: "Profissionais de nível médio de enfermagem",
    3222: "Profissionais de nível médio de partos",
    3230: "Profissionais de nível médio de medicina tradicional e alternativa",
    3240: "Técnicos e assistentes veterinários",
    3251: "Dentistas auxiliares e ajudantes de odontologia",
    3252: "Técnicos em documentação sanitária",
    3253: "Trabalhadores comunitários da saúde",
    3254: "Técnicos em optometria e ópticos",
    3255: "Técnicos e assistentes fisioterapeutas",
    3256: " assistentes de medicina",
    3257: "Inspetores de saúde laboral, ambiental e afins",
    3258: "Ajudantes de ambulâncias",
    3259: "Profissionais de nível médio da saúde não classificados anteriormente",
    3311: "Agentes e corretores de bolsa, câmbio e outros serviços financeiros",
    3312: "Agentes de empréstimos e financiamento",
    3313: "Contabilistas e guarda livros",
    3314: "Profissionais de nível médio de serviços estatísticos, matemáticos e afins",
    3315: "Avaliadores",
    3321: "Agentes de seguros",
    3322: "Representantes comerciais",
    3323: "Agentes de compras",
    3324: "Corretores de comercialização",
    3331: "Despachantes aduaneiros",
    3332: "Organizadores de conferências e eventos",
    3333: "Agentes de emprego e agenciadores de mão de obra",
    3334: "Agentes imobiliários",
    3339: "Agentes de serviços comerciais não classificados anteriormente",
    3341: "Supervisores de secretaria",
    3342: "Secretários jurídicos",
    3343: "Secretários executivos e administrativos",
    3344: "Secretários de medicina",
    3351: "Agentes aduaneiros e inspetores de fronteiras",
    3352: "Agentes da administração tributária",
    3353: "Agentes de serviços de seguridade social",
    3354: "Agentes de serviços de expedição de licenças e permissões",
    3355: "Inspetores de polícia e detetives",
    3359: "Agentes da administração pública para aplicação da lei e afins não classificados anteriormente",
    3411: "Profissionais de nível médio do direito e serviços legais e afins",
    3412: "Trabalhadores e assistentes sociais de nível médio",
    3413: "Auxiliares leigos de religião",
    3421: "Atletas e esportistas",
    3422: "Treinadores, instrutores e árbitros de atividades esportivas",
    3423: "Instrutores de educação física e atividades recreativas",
    3431: "Fotógrafos",
    3432: "Desenhistas e decoradores de interiores",
    3433: "Técnicos em galerias de arte, museus e bibliotecas",
    3434: "Chefes de cozinha",
    3435: "Outros profissionais de nível médio em atividades culturais e artísticas",
    3511: "Técnicos em operações de tecnologia da informação e das comunicações",
    3512: "Técnicos em assistência ao usuário de tecnologia da informação e das comunicações",
    3513: "Técnicos de redes e sistemas de computadores",
    3514: "Técnicos da web",
    3521: "Técnicos de radiodifusão e gravação audiovisual",
    3522: "Técnicos de engenharia de telecomunicações",
    4110: "Escriturários gerais",
    4120: "Secretários (geral)",
    4131: "Operadores de máquinas de processamento de texto e mecanógrafos",
    4132: "Operadores de entrada de dados",
    4211: "Caixas de banco e afins",
    4212: "Coletores de apostas e de jogos",
    4213: "Trabalhadores em escritórios de empréstimos e penhor",
    4214: "Cobradores e afins",
    4221: "Trabalhadores de agências de viagem",
    4222: "Trabalhadores de centrais de atendimento",
    4223: "Telefonistas",
    4224: "Recepcionistas de hotéis",
    4225: "Trabalhadores dos serviços de informações",
    4226: "Recepcionistas em geral",
    4227: "Entrevistadores de pesquisas de mercado",
    4229: "Trabalhadores de serviços de informação ao cliente não classificados anteriormente",
    4311: "Trabalhadores de contabilidade e cálculo de custos",
    4312: "Trabalhadores de serviços estatísticos, financeiros e de seguros",
    4313: "Trabalhadores encarregados de folha de pagamento",
    4321: "Trabalhadores de controle de abastecimento e estoques",
    4322: "Trabalhadores de serviços de apoio à produção",
    4323: "Trabalhadores de serviços de transporte",
    4411: "Trabalhadores de bibliotecas",
    4412: "Trabalhadores de serviços de correios",
    4413: "Codificadores de dados, revisores de provas de impressão e afins",
    4414: "Outros escreventes",
    4415: "Trabalhadores de arquivos",
    4416: "Trabalhadores do serviço de pessoal",
    4419: "Trabalhadores de apoio administrativo não classificados anteriormente",
    5111: "Auxiliares de serviço de bordo",
    5112: "Fiscais e cobradores de transportes públicos",
    5113: "Guias de turismo",
    5120: "Cozinheiros",
    5131: "Garçons",
    5132: " atendentes de bar",
    5141: "Cabeleireiros",
    5142: "Especialistas em tratamento de beleza e afins",
    5151: "Supervisores de manutenção e limpeza de edifícios em escritórios, hotéis e estabelecimentos",
    5152: "Governantas e mordomos domésticos",
    5153: "Porteiros e zeladores",
    5161: "Astrólogos, adivinhos e afins",
    5162: "Acompanhantes e criados particulares",
    5163: "Trabalhadores de funerárias e embalsamadores",
    5164: "Cuidadores de animais",
    5165: "Instrutores de autoescola",
    5168: "Trabalhadores do sexo",
    5169: "Trabalhadores de serviços pessoais não classificados anteriormente",
    5211: "Vendedores de quiosques e postos de mercados",
    5212: "Vendedores ambulantes de serviços de alimentação",
    5221: "Comerciantes de lojas",
    5222: "Supervisores de lojas",
    5223: "Balconistas e vendedores de lojas",
    5230: "Caixas e expedidores de bilhetes",
    5241: "Modelos de moda, arte e publicidade",
    5242: "Demonstradores de lojas",
    5243: "Vendedores a domicilio",
    5244: "Vendedores por telefone",
    5245: "Frentistas de posto de gasolina",
    5246: "Balconistas dos serviços de alimentação",
    5249: "Vendedores não classificados anteriormente",
    5311: "Cuidadores de crianças",
    5312: "Ajudantes de professores",
    5321: "Trabalhadores de cuidados pessoais em instituições",
    5322: "Trabalhadores de cuidados pessoais a domicílios",
    5329: "Trabalhadores de cuidados pessoais nos serviços de saúde não classificados anteriormente",
    5411: "Bombeiros",
    5412: "Policiais",
    5413: "Guardiões de presídios",
    5414: "Guardas de segurança",
    5419: "Trabalhadores dos serviços de proteção e segurança não classificados anteriormente",
    6111: "Agricultores e trabalhadores qualificados em atividades da agricultura (exclusive hortas, viveiros e jardins)",
    6112: "Agricultores e trabalhadores qualificados no cultivo de hortas, viveiros e jardins",
    6114: "Agricultores e trabalhadores qualificados de cultivos mistos",
    6121: "Criadores de gado e trabalhadores qualificados da criação de gado",
    6122: "Avicultores e trabalhadores qualificados da avicultura",
    6123: "Apicultores, sericicultores e trabalhadores qualificados da apicultura e sericicultura",
    6129: "Outros criadores e trabalhadores qualificados da pecuária não classificados anteriormente",
    6130: "Produtores e trabalhadores qualificados de exploração agropecuária mista",
    6210: "Trabalhadores florestais qualificados e afins",
    6221: "Trabalhadores da aquicultura",
    6224: "Caçadores",
    6225: "Pescadores",
    7111: "Construtores de casas",
    7112: "Pedreiros",
    7113: "Canteiros, cortadores e gravadores de pedras",
    7114: "Trabalhadores em cimento e concreto armado",
    7115: "Carpinteiros",
    7119: "Outros trabalhadores qualificados e operários da construção não classificados anteriormente",
    7121: "Telhadores",
    7122: "Aplicadores de revestimentos cerâmicos, pastilhas, pedras e madeiras",
    7123: "Gesseiros",
    7124: "Instaladores de material isolante térmico e acústico",
    7125: "Vidraceiros",
    7126: "Bombeiros e encanadores",
    7127: "Mecânicos-instaladores de sistemas de refrigeração e climatização",
    7131: "Pintores e empapeladores",
    7132: "Lustradores",
    7133: "Limpadores de fachadas",
    7211: "Moldadores de metal e macheiros",
    7212: "Soldadores e oxicortadores",
    7213: "Chapistas e caldeireiros",
    7214: "Montadores de estruturas metálicas",
    7215: "Aparelhadores e emendadores de cabos",
    7221: "Ferreiros e forjadores",
    7222: "Ferramenteiros e afins",
    7223: "Reguladores e operadores de máquinas-ferramentas",
    7224: "Polidores de metais e afiadores de ferramentas",
    7231: "Mecânicos e reparadores de veículos a motor",
    7232: "Mecânicos e reparadores de motores de avião",
    7233: "Mecânicos e reparadores de máquinas agrícolas e industriais",
    7234: "Reparadores de bicicletas e afins",
    7311: "Mecânicos e reparadores de instrumentos de precisão",
    7312: "Confeccionadores e afinadores de instrumentos musicais",
    7313: "Joalheiros e lapidadores de gemas, artesãos de metais preciosos e semipreciosos",
    7314: "Ceramistas e afins (preparação e fabricação)",
    7315: "Cortadores, polidores, jateadores e gravadores de vidros e afins",
    7316: "Redatores de cartazes, pintores decorativos e gravadores",
    7317: "Artesãos de pedra, madeira, vime e materiais semelhantes",
    7318: "Artesãos de tecidos, couros e materiais semelhantes",
    7319: "Artesãos não classificados anteriormente",
    7321: "Trabalhadores da pré-impressão gráfica",
    7322: "Impressores",
    7323: "Encadernadores e afins",
    7411: "Eletricistas de obras e afins",
    7412: "Mecânicos e ajustadores eletricistas",
    7413: "Instaladores e reparadores de linhas elétricas",
    7421: "Mecânicos e reparadores em eletrônica",
    7422: "Instaladores e reparadores em tecnologias da informação e comunicações",
    7511: "Magarefes e afins",
    7512: "Padeiros, confeiteiros e afins",
    7513: "Trabalhadores da pasteurização do leite e fabricação de laticínios e afins",
    7514: "Trabalhadores da conservação de frutas, legumes e similares",
    7515: "Trabalhadores da degustação e classificação de alimentos e bebidas",
    7516: "Trabalhadores qualificados da preparação do fumo e seus produtos",
    7521: "Trabalhadores de tratamento e preparação da madeira",
    7522: "Marceneiros e afins",
    7523: "Operadores de máquinas de lavrar madeira",
    7531: "Alfaiates, modistas, chapeleiros e peleteiros",
    7532: "Trabalhadores qualificados da preparação da confecção de roupas",
    7533: "Costureiros, bordadeiros e afins",
    7534: "Tapeceiros, colchoeiros e afins",
    7535: "Trabalhadores qualificados do tratamento de couros e peles",
    7536: "Sapateiros e afins",
    7541: "Trabalhadores subaquáticos",
    7542: "Dinamitadores e detonadores",
    7543: "Classificadores e provadores de produtos (exceto de bebidas e alimentos)",
    7544: "Fumigadores e outros controladores de pragas e ervas daninhas",
    7549: "Outros trabalhadores qualificados e operários da indústria e do artesanato não classificados anteriormente",
    8111: "Mineiros e operadores de máquinas e de instalações em minas e pedreiras",
    8112: "Operadores de instalações de processamento de minerais e rochas",
    8113: "Perfuradores e sondadores de poços e afins",
    8114: "Operadores de máquinas para fabricar cimento, pedras e outros produtos minerais",
    8121: "Operadores de instalações de processamento de metais",
    8122: "Operadores de máquinas polidoras, galvanizadoras e recobridoras de metais",
    8131: "Operadores de instalações e máquinas de produtos químicos",
    8132: "Operadores de máquinas para fabricar produtos fotográficos",
    8141: "Operadores de máquinas para fabricar produtos de borracha",
    8142: "Operadores de máquinas para fabricar produtos de material plástico",
    8143: "Operadores de máquinas para fabricar produtos de papel",
    8151: "Operadores de máquinas de preparação de fibras, fiação e bobinamento de fios",
    8152: "Operadores de teares e outras máquinas de tecelagem",
    8153: "Operadores de máquinas de costura",
    8154: "Operadores de máquinas de branqueamento, tingimento e limpeza de tecidos",
    8155: "Operadores de máquinas de processamento de couros e peles",
    8156: "Operadores de máquinas para fabricação de calçados e afins",
    8157: "Operadores de máquinas de lavar, tingir e passar roupas",
    8159: "Operadores de máquinas para fabricar produtos têxteis e artigos de couro e pele não classificados anteriormente",
    8160: "Operadores de máquinas para elaborar alimentos e produtos afins",
    8171: "Operadores de instalações para a preparação de pasta de papel e papel",
    8172: "Operadores de instalações para processamento de madeira",
    8181: "Operadores de instalações de vidraria e cerâmica",
    8182: "Operadores de máquinas de vapor e caldeiras",
    8183: "Operadores de máquinas de embalagem, engarrafamento e etiquetagem",
    8189: "Operadores de máquinas e de instalações fixas não classificados anteriormente",
    8211: "Mecânicos montadores de maquinaria mecânica",
    8212: "Montadores de equipamentos elétricos e eletrônicos",
    8219: "Montadores não classificados anteriormente",
    8311: "Maquinistas de locomotivas",
    8312: "Guarda-freios e agentes de manobras",
    8321: "Condutores de motocicletas",
    8322: "Condutores de automóveis, taxis e caminhonetes",
    8331: "Condutores de ônibus e bondes",
    8332: "Condutores de caminhões pesados",
    8341: "Operadores de máquinas agrícolas e florestais móveis",
    8342: "Operadores de máquinas de movimentação de terras e afins",
    8343: "Operadores de guindastes, gruas, aparatos de elevação e afins",
    8344: "Operadores de empilhadeiras",
    8350: "Marinheiros de coberta e afins",
    9111: "Trabalhadores dos serviços domésticos em geral",
    9112: "Trabalhadores de limpeza de interior de edifícios, escritórios, hotéis e outros estabelecimentos",
    9121: "Lavadeiros de roupas e passadeiros manuais",
    9122: "Lavadores de veículos",
    9123: "Limpadores de janelas",
    9129: "Outros trabalhadores de limpeza",
    9211: "Trabalhadores elementares da agricultura",
    9212: "Trabalhadores elementares da pecuária",
    9213: "Trabalhadores elementares da agropecuária",
    9214: "Trabalhadores elementares da jardinagem e horticultura",
    9215: "Trabalhadores florestais elementares",
    9216: "Trabalhadores elementares da pesca e aquicultura",
    9311: "Trabalhadores elementares de minas e pedreiras",
    9312: "Trabalhadores elementares de obras públicas e da manutenção de estradas, represas e similares",
    9313: "Trabalhadores elementares da construção de edifícios",
    9321: " empacotadores manuais",
    9329: "Trabalhadores elementares da indústria de transformação não classificados anteriormente",
    9331: "Condutores de veículos acionados a pedal ou a braços",
    9332: "Condutores de veículos e máquinas de tração animal",
    9333: "Carregadores",
    9334: "Repositores de prateleiras",
    9411: "Preparadores de comidas rápidas",
    9412: "Ajudantes de cozinha",
    9510: "Trabalhadores ambulantes dos serviços e afins",
    9520: "Vendedores ambulantes (exclusive de serviços de alimentação)",
    9611: "Coletores de lixo e material reciclável",
    9612: "Classificadores de resíduos",
    9613: "Varredores e afins",
    9621: "Mensageiros, carregadores de bagagens e entregadores de encomendas",
    9622: "Pessoas que realizam várias tarefas",
    9623: "Coletores de dinheiro em máquinas automáticas de venda e leitores de medidores",
    9624: "Carregadores de água e coletores de lenha",
    9629: "Outras ocupações elementares não classificadas anteriormente",
    110: "Oficiais das forças armadas",
    210: "Graduados e praças das forças armadas",
    411: "Oficiais de polícia militar",
    412: "Graduados e praças da polícia militar",
    511: "Oficiais de bombeiro militar",
    512: "Graduados e praças do corpo de bombeiros"
}

CNAE_ACTIVITY = {
    1101: "Cultivo de arroz",
    1102: "Cultivo de milho",
    1103: "Cultivo de outros cereais",
    1104: "Cultivo de algodão",
    1105: "Cultivo de cana-de-açúcar",
    1106: "Cultivo de fumo",
    1107: "Cultivo de soja",
    1108: "Cultivo de mandioca",
    1109: "Cultivo de outras lavouras temporárias não especificadas anteriormente",
    1110: "Horticultura",
    1111: "Cultivo de flores e plantas ornamentais",
    1112: "Cultivo de frutas cítricas",
    1113: "Cultivo de café",
    1114: "Cultivo de cacau",
    1115: "Cultivo de uva",
    1116: "Cultivo de banana",
    1117: "Cultivo de outras plantas e frutas de lavoura permanente não especificadas anteriormente",
    1118: "Produção de sementes e mudas certificadas",
    1119: "Lavoura não especificada",
    1201: "Criação de bovinos",
    1202: "Criação de outros animais de grande porte não especificados anteriormente",
    1203: "Criação de caprinos e ovinos",
    1204: "Criação de suínos",
    1205: "Criação de aves",
    1206: "Apicultura",
    1207: "Sericicultura",
    1208: "Criação de outros animais não especificados anteriormente",
    1209: "Pecuária não especificada",
    1401: "Atividades de apoio à agricultura e pós-colheita",
    1402: "Atividades de apoio à pecuária",
    1500: "Caça e serviços relacionados",
    1999: "Agropecuária",
    2000: "Produção florestal",
    3001: "Pesca",
    3002: "Aqüicultura",
    5000: "Extração de carvão mineral",
    6000: "Extração de petróleo e gás natural",
    7001: "Extração de minérios de metais preciosos",
    7002: "Extração de minerais metálicos não especificados anteriormente",
    8001: "Extração de pedras, areia e argila",
    8002: "Extração de gemas (pedras preciosas e semi-preciosas)",
    8009: "Extração de minerais não metálicos não especificados anteriormente",
    9000: "Atividades de apoio à extração de minerais",
    10010: "Abate e fabricação de produtos de carne e pescado",
    10021: "Fabricação de conservas de frutas, legumes e outros vegetais",
    10022: "Fabricação de óleos vegetais e gorduras vegetais e animais",
    10030: "Laticínios",
    10091: "Moagem, fabricação de produtos amiláceos e de alimentos para animais",
    10092: "Fabricação e refino do açúcar",
    10093: "Torrefação e moagem de café",
    10099: "Fabricação de outros produtos alimentícios",
    11000: "Fabricação de bebidas",
    12000: "Processamento industrial e fabricação de produtos do fumo",
    13001: "Preparação de fibras, fiação e tecelagem",
    13002: "Fabricação de artefatos têxteis, exceto vestuário",
    14001: "Confecção de artigos do vestuário e acessórios, exceto sob medida",
    14002: "Confecção, sob medida, de artigos do vestuário",
    15011: "Curtimento e outras preparações de couro",
    15012: "Fabricação de artigos de viagem e de artefatos diversos de couro",
    15020: "Fabricação de calçados e partes para calçados, de qualquer material",
    16001: "Serrarias",
    16002: "Fabricação de produtos de madeira, cortiça e material trançado, exceto móveis",
    17001: "Fabricação de celulose, papel, cartolina e papel-cartão",
    17002: "Fabricação de embalagens e de produtos diversos de papel, cartolina, papel-cartão e papelão ondulado",
    18000: "Impressão e reprodução de gravações",
    19010: "Coquerias",
    19020: "Fabricação de produtos derivados do petróleo",
    19030: "Produção de biocombustíveis",
    20010: "Fabricação de tintas, vernizes, esmaltes, lacas e produtos afins",
    20020: "Fabricação de sabões, detergentes, produtos de limpeza, cosméticos, produtos de perfumaria e de higiene pessoal",
    20090: "Fabricação de outros produtos químicos não especificados anteriormente",
    21000: "Fabricação de produtos farmoquimicos e farmacêuticos",
    22010: "Fabricação de produtos de borracha",
    22020: "Fabricação de produtos de material plástico",
    23010: "Fabricação de vidro e produtos de vidro",
    23091: "Fabricação de produtos cerâmicos",
    23099: "Fabricação de outros produtos de minerais não-metálicos não especificados anteriormente",
    24001: "Fabricação de produtos siderúrgicos",
    24002: "Metalurgia dos metais não-ferrosos",
    24003: "Fundição",
    25001: "Fabricação de produtos de metal, exceto máquinas e equipamentos",
    25002: "Forjaria, estamparia, metalurgia do pó e serviços de tratamento de metais",
    26010: "Fabricação de componentes eletrônicos",
    26020: "Fabricação de equipamentos de informática e periféricos",
    26030: "Fabricação de equipamentos de comunicação e de aparelhos de recepção, reprodução, gravação e amplificação de áudio e vídeo",
    26041: "Fabricação de aparelhos e instrumentos de medida, teste e controle; cronômetros e relógios e de aparelhos eletromédicos e eletroterapêuticos",
    26042: "Fabricação de equipamentos e instrumentos ópticos, fotográficos e cinematográficos e de mídias virgens, magnéticas e ópticas",
    27010: "Fabricação de eletrodomesticos",
    27090: "Fabricação de máquinas, aparelhos e materiais elétricos diversos, exceto eletrodomesticos",
    28000: "Fabricação de máquinas e equipamentos",
    29001: "Fabricação e montagem de veículos automotores",
    29002: "Fabricação de cabines, carrocerias, reboques e peças para veículos automotores",
    29003: "Reconstrução, em fábrica, de motores de veículos automotores",
    30010: "Construção de embarcações",
    30020: "Fabricação de veículos ferroviários",
    30030: "Fabricação de aeronaves",
    30090: "Fabricação de outros equipamentos de transporte não especificados anteriormente",
    31000: "Fabricação de móveis",
    32001: "Fabricação de artigos de joalheria, bijuteria e semelhantes",
    32002: "Fabricação de instrumentos musicais",
    32003: "Fabricação de artefatos para pesca e esporte e de brinquedos e jogos recreativos",
    32009: "Fabricação de produtos diversos",
    33001: "Manutenção e reparação de máquinas e equipamentos",
    33002: "Instalação de máquinas e equipamentos",
    35010: "Geração, transmissão e distribuição de energia elétrica",
    35021: "Produção e distribuição de combustíveis gasosos por redes urbanas",
    35022: "Produção e distribuição de vapor, água quente e ar condicionado",
    36000: "Captação, tratamento e distribuição de água",
    37000: "Esgoto e atividades relacionadas",
    38000: "Coleta, tratamento e disposição de resíduos; recuperação de materiais",
    39000: "Descontaminação e outros serviços de gestão de resíduos",
    41000: "Construção de edifícios",
    42000: "Construção de obras de infra-estrutura",
    43000: "Serviços especializados para construção",
    45010: "Comércio de veículos automotores",
    45020: "Manutenção e reparação de veículos automotores",
    45030: "Comércio de peças e acessórios para veículos automotores",
    45040: "Comércio, manutenção e reparação de motocicletas, peças e acessórios",
    48010: "Representantes comerciais e agentes do comércio, exceto de veículos automotores e motocicletas",
    48020: "Comércio de matérias-primas agrícolas e animais vivos",
    48030: "Comércio de produtos alimentícios, bebidas e fumo",
    48041: "Comércio de tecidos, artefatos de tecidos e armarinho",
    48042: "Comércio de artigos do vestuário, complementos, calçados e artigos de viagem",
    48050: "Comércio de madeira, material de construção, ferragens e ferramentas",
    48060: "Comércio de combustíveis para veículos automotores",
    48071: "Comércio de produtos farmaceuticos, médicos, ortopédicos, odontológicos e de cosméticos e perfumaria",
    48072: "Comércio de artigos de escritório e de papelaria; livros, jornais e outras publicações",
    48073: "Comércio de eletrodomésticos, móveis e outros artigos de residência",
    48074: "Comércio de equipamentos e produtos de tecnologias de informação e comunicação",
    48075: "Comércio de máquinas, aparelhos e equipamentos, exceto eletrodomésticos",
    48076: "Comércio de combustíveis sólidos, líquidos e gasosos, exceto para veículos automotores",
    48077: "Comércio de produtos usados",
    48078: "Comercio de residuos e sucatas",
    48079: "Comércio de produtos novos não especificados anteriormente",
    48080: "Supermercado e hipermercado",
    48090: "Lojas de departamento e outros comércios não especializados, sem predominância de produtos alimentícios",
    48100: "Comércio ambulante e feiras",
    49010: "Transporte ferroviário e metroferroviário",
    49030: "Transporte rodoviário de passageiros",
    49040: "Transporte rodoviário de carga",
    49090: "Outros transportes terrestres",
    50000: "Transporte Aquaviário",
    51000: "Transporte Aéreo",
    52010: "Armazenamento, carga e descarga",
    52020: "Atividades auxiliares dos transportes e atividades relacionadas à organização do transporte de carga",
    53001: "Atividades de Correio",
    53002: "Atividades de malote e de entrega",
    55000: "Alojamento",
    56011: "Restaurantes e outros estabelecimentos de serviços de alimentação e bebidas",
    56012: "Serviços de catering, bufê e outros serviços de comida preparada",
    56020: "Serviços ambulantes de alimentação",
    58000: "Edição e Edição integrada à impressão",
    59000: "Atividades cinematográficas, produção de vídeos e de programas de televisão, gravação de som e de música",
    60001: "Atividades de rádio",
    60002: "Atividades de televisão",
    61000: "Telecomunicações",
    62000: "Atividades dos serviços de tecnologia da informação",
    63000: "Atividades de prestação de serviços de informação",
    64000: "Serviços financeiros",
    65000: "Seguros e previdência privada",
    66001: "Atividades auxiliares dos serviços financeiros",
    66002: "Atividades auxiliares dos seguros, da previdência complementar e dos planos de saúde",
    68000: "Atividades imobiliárias",
    69000: "Atividades jurídicas, de contabilidade e de auditoria",
    70000: "Atividades de consultoria em gestão empresarial",
    71000: "Serviços de arquitetura e engenharia e atividades técnicas relacionadas; Testes e análises técnicas",
    72000: "Pesquisa e desenvolvimento científico",
    73010: "Publicidade",
    73020: "Pesquisas de mercado e opinião pública",
    74000: "Outras atividades profissionais, científicas e técnicas não especificadas anteriormente",
    75000: "Atividades veterinárias",
    77010: "Aluguel de objetos pessoais e domésticos",
    77020: "Aluguel de meios de transportes, maquinas e equipamentos sem operador e gestão de ativos intangíveis não financeiros",
    78000: "Seleção, agenciamento e locação de mão-de-obra",
    79000: "Agências de viagens, operadores turísticos e serviços de reservas",
    80000: "Atividades de vigilância, segurança, transporte de valores e investigação",
    81011: "Serviços de limpeza e de apoio a edifícios, exceto condomínios prediais",
    81012: "Condomínios prediais",
    81020: "Atividades paisagísticas",
    82001: "Serviços de escritório e apoio administrativo",
    82002: "Atividades de teleatendimento",
    82003: "Atividades de organização de eventos, exceto culturais e esportivos",
    82009: "Outras atividades de serviços prestados principalmente às empresas",
    84011: "Administração publica e regulação da política econômica e social - Federal",
    84012: "Administração publica e regulação da política econômica e social - Estadual",
    84013: "Administração publica e regulação da política econômica e social - Municipal",
    84014: "Defesa",
    84015: "Outros serviços coletivos prestados pela administração pública - Federal",
    84016: "Outros serviços coletivos prestados pela administração pública - Estadual",
    84017: "Outros serviços coletivos prestados pela administração pública - Municipal",
    84020: "Seguridade social obrigatória",
    85011: "Creche",
    85012: "Pré-escola e ensino fundamental",
    85013: "Ensino médio",
    85014: "Educação superior",
    85021: "Serviços auxiliares à educação",
    85029: "Outras atividades de ensino",
    86001: "Atividades de atendimento hospitalar",
    86002: "Atividades de atenção ambulatorial executadas por médicos e odontólogos",
    86003: "Atividades de serviços de complementação diagnóstica e terapêutica",
    86004: "Atividades de profissionais da área de saúde, exceto médicos e odontólogos",
    86009: "Atividades de atenção à saúde humana não especificadas anteriormente",
    87000: "Atividades de assistência à saude humana integradas com assistencia social, inclusive prestadas em residencias",
    88000: "Serviços de assistência social sem alojamento",
    90000: "Atividades artísticas, criativas e de espetáculos",
    91000: "Atividades ligadas ao patrimônio cultural e ambiental",
    92000: "Atividades de exploração de jogos de azar e apostas",
    93011: "Atividades esportivas",
    93012: "Atividades de condicionamento físico",
    93020: "Atividades de recreação e lazer",
    94010: "Atividades de organizações associativas patronais, empresariais e profissionais",
    94020: "Atividades de organizações sindicais",
    94091: "Atividades de organizações religiosas e filosóficas",
    94099: "Outras atividades associativas não especificadas anteriormente",
    95010: "Reparação e manutenção de equipamentos de informática e comunicação",
    95030: "Reparação e manutenção de objetos e equipamentos pessoais e domésticos",
    96010: "Lavanderias, tinturarias e toalheiros",
    96020: "Cabeleireiros e outras atividades de tratamento de beleza",
    96030: "Atividades funerárias e serviços relacionados",
    96090: "Outras atividades de serviços pessoais",
    97000: "Serviços domésticos",
    99000: "Organismos internacionais e outras instituições extraterritoriais",
    0: "Atividades mal definidas"
}

SCHEMA = {
    "Ano": "Int64",
    "Trimestre": "Int64",
    "UF": "Int64",
    "Capital": "Int64",
    "RM_RIDE": "Int64",
    "V1022": "Int64",
    "V1023": "Int64",
    "V1028": "Float64",
    "V2001": "Int64",
    "V2005": "Int64",
    "V2007": "Int64",
    "V2008": "Int64",
    "V20081": "Int64",
    "V20082": "Int64",
    "V2009": "Int64",
    "V2010": "Int64",
    "V3001": "Int64",
    "V3002": "Int64",
    "V3002A": "Int64",
    "V3003": "Int64",
    "V3003A": "Int64",
    "V3004": "Int64",
    "V3005": "Int64",
    "V3005A": "Int64",
    "V3006": "Int64",
    "V3006A": "Int64",
    "V3007": "Int64",
    "V3008": "Int64",
    "V3009": "Int64",
    "V3009A": "Int64",
    "V3010": "Int64",
    "V3011": "Int64",
    "V3011A": "Int64",
    "V3012": "Int64",
    "V3013": "Int64",
    "V3013A": "Int64",
    "V3013B": "Int64",
    "V3014": "Int64",
    "V4001": "Int64",
    "V4002": "Int64",
    "V4003": "Int64",
    "V4004": "Int64",
    "V4005": "Int64",
    "V4006": "Int64",
    "V4006A": "Int64",
    "V4007": "Int64",
    "V4008": "Int64",
    "V40081": "Int64",
    "V40082": "Int64",
    "V40083": "Int64",
    "V4009": "Int64",
    "V4010": "Int64",
    "V4012": "Int64",
    "V40121": "Int64",
    "V4013": "Int64",
    "V40132": "Int64",
    "V40132A": "Int64",
    "V4014": "Int64",
    "V4015": "Int64",
    "V40151": "Int64",
    "V401511": "Int64",
    "V401512": "Int64",
    "V4016": "Int64",
    "V40161": "Int64",
    "V40162": "Int64",
    "V40163": "Int64",
    "V4017": "Int64",
    "V40171": "Int64",
    "V401711": "Int64",
    "V4018": "Int64",
    "V40181": "Int64",
    "V40182": "Int64",
    "V40183": "Int64",
    "V4019": "Int64",
    "V4020": "Int64",
    "V4021": "Int64",
    "V4022": "Int64",
    "V4024": "Int64",
    "V4025": "Int64",
    "V4026": "Int64",
    "V4027": "Int64",
    "V4028": "Int64",
    "V4029": "Int64",
    "V4032": "Int64",
    "V4033": "Int64",
    "V40331": "Int64",
    "V403311": "Int64",
    "V403312": "Float64",
    "V40332": "Int64",
    "V403321": "Int64",
    "V403322": "Float64",
    "V40333": "Int64",
    "V403331": "Int64",
    "V4034": "Int64",
    "V40341": "Int64",
    "V403411": "Int64",
    "V403412": "Float64",
    "V40342": "Int64",
    "V403421": "Int64",
    "V403422": "Float64",
    "V4039": "Int64",
    "V4039C": "Int64",
    "V4040": "Int64",
    "V40401": "Int64",
    "V40402": "Int64",
    "V40403": "Int64",
    "V4041": "Int64",
    "V4043": "Int64",
    "V40431": "Int64",
    "V4044": "Int64",
    "V4045": "Int64",
    "V4046": "Int64",
    "V4047": "Int64",
    "V4048": "Int64",
    "V4049": "Int64",
    "V4050": "Int64",
    "V40501": "Int64",
    "V405011": "Int64",
    "V405012": "Float64",
    "V40502": "Int64",
    "V405021": "Int64",
    "V405022": "Float64",
    "V40503": "Int64",
    "V405031": "Int64",
    "V4051": "Int64",
    "V40511": "Int64",
    "V405111": "Int64",
    "V405112": "Float64",
    "V40512": "Int64",
    "V405121": "Int64",
    "V405122": "Float64",
    "V4056": "Int64",
    "V4056C": "Int64",
    "V4057": "Int64",
    "V4058": "Int64",
    "V40581": "Int64",
    "V405811": "Int64",
    "V405812": "Float64",
    "V40582": "Int64",
    "V405821": "Int64",
    "V405822": "Float64",
    "V40583": "Int64",
    "V405831": "Int64",
    "V40584": "Int64",
    "V4059": "Int64",
    "V40591": "Int64",
    "V405911": "Int64",
    "V405912": "Float64",
    "V40592": "Int64",
    "V405921": "Int64",
    "V405922": "Float64",
    "V4062": "Int64",
    "V4062C": "Int64",
    "V4063": "Int64",
    "V4063A": "Int64",
    "V4064": "Int64",
    "V4064A": "Int64",
    "V4071": "Int64",
    "V4072": "Int64",
    "V4072A": "Int64",
    "V4073": "Int64",
    "V4074": "Int64",
    "V4074A": "Int64",
    "V4075A": "Int64",
    "V4075A1": "Int64",
    "V4076": "Int64",
    "V40761": "Int64",
    "V40762": "Int64",
    "V40763": "Int64",
    "V4077": "Int64",
    "V4078": "Int64",
    "V4078A": "Int64",
    "V4082": "Int64",
    "VD2002": "Int64",
    "VD2003": "Int64",
    "VD2004": "Int64",
    "VD2006": "Int64",
    "VD3004": "Int64",
    "VD3005": "Int64",
    "VD3006": "Int64",
    "VD4001": "Int64",
    "VD4002": "Int64",
    "VD4003": "Int64",
    "VD4004": "Int64",
    "VD4004A": "Int64",
    "VD4005": "Int64",
    "VD4007": "Int64",
    "VD4008": "Int64",
    "VD4009": "Int64",
    "VD4010": "Int64",
    "VD4011": "Int64",
    "VD4012": "Int64",
    "VD4013": "Int64",
    "VD4014": "Int64",
    "VD4015": "Int64",
    "VD4016": "Float64",
    "VD4017": "Float64",
    "VD4018": "Int64",
    "VD4019": "Float64",
    "VD4020": "Float64",
    "VD4023": "Int64",
    "VD4030": "Int64",
    "VD4031": "Int64",
    "VD4032": "Int64",
    "VD4033": "Int64",
    "VD4034": "Int64",
    "VD4035": "Int64",
    "VD4036": "Int64",
    "VD4037": "Int64"
}

COLUMN_WIDTHS = {
    "Ano": [0, 4],
    "Trimestre": [4, 5],
    "UF": [5, 7],
    "Capital": [7, 9],
    "RM_RIDE": [9, 11],
    "V1022": [32, 33],
    "V1023": [33, 34],
    "V1028": [49, 64],
    "V2001": [88, 90],
    "V2005": [92, 94],
    "V2007": [94, 95],
    "V2008": [95, 97],
    "V20081": [97, 99],
    "V20082": [99, 103],
    "V2009": [103, 106],
    "V2010": [106, 107],
    "V3001": [107, 108],
    "V3002": [108, 109],
    "V3002A": [109, 110],
    "V3003": [110, 112],
    "V3003A": [112, 114],
    "V3004": [114, 115],
    "V3005": [115, 116],
    "V3005A": [116, 117],
    "V3006": [117, 119],
    "V3006A": [119, 120],
    "V3007": [120, 121],
    "V3008": [121, 122],
    "V3009": [122, 124],
    "V3009A": [124, 126],
    "V3010": [126, 127],
    "V3011": [127, 128],
    "V3011A": [128, 129],
    "V3012": [129, 130],
    "V3013": [130, 132],
    "V3013A": [132, 133],
    "V3013B": [133, 134],
    "V3014": [134, 135],
    "V4001": [135, 136],
    "V4002": [136, 137],
    "V4003": [137, 138],
    "V4004": [138, 139],
    "V4005": [139, 140],
    "V4006": [140, 141],
    "V4006A": [141, 142],
    "V4007": [142, 143],
    "V4008": [143, 144],
    "V40081": [144, 146],
    "V40082": [146, 148],
    "V40083": [148, 150],
    "V4009": [150, 151],
    "V4010": [151, 155],
    "V4012": [155, 156],
    "V40121": [156, 157],
    "V4013": [157, 162],
    "V40132": [162, 163],
    "V40132A": [163, 164],
    "V4014": [164, 165],
    "V4015": [165, 166],
    "V40151": [166, 167],
    "V401511": [167, 168],
    "V401512": [168, 170],
    "V4016": [170, 171],
    "V40161": [171, 172],
    "V40162": [172, 174],
    "V40163": [174, 176],
    "V4017": [176, 177],
    "V40171": [177, 178],
    "V401711": [178, 179],
    "V4018": [179, 180],
    "V40181": [180, 181],
    "V40182": [181, 183],
    "V40183": [183, 185],
    "V4019": [185, 186],
    "V4020": [186, 187],
    "V4021": [187, 188],
    "V4022": [188, 189],
    "V4024": [189, 190],
    "V4025": [190, 191],
    "V4026": [191, 192],
    "V4027": [192, 193],
    "V4028": [193, 194],
    "V4029": [194, 195],
    "V4032": [195, 196],
    "V4033": [196, 197],
    "V40331": [197, 198],
    "V403311": [198, 199],
    "V403312": [199, 207],
    "V40332": [207, 208],
    "V403321": [208, 209],
    "V403322": [209, 217],
    "V40333": [217, 218],
    "V403331": [218, 219],
    "V4034": [219, 220],
    "V40341": [220, 221],
    "V403411": [221, 222],
    "V403412": [222, 230],
    "V40342": [230, 231],
    "V403421": [231, 232],
    "V403422": [232, 240],
    "V4039": [240, 243],
    "V4039C": [243, 246],
    "V4040": [246, 247],
    "V40401": [247, 249],
    "V40402": [249, 251],
    "V40403": [251, 253],
    "V4041": [253, 257],
    "V4043": [257, 258],
    "V40431": [258, 259],
    "V4044": [259, 264],
    "V4045": [264, 265],
    "V4046": [265, 266],
    "V4047": [266, 267],
    "V4048": [267, 268],
    "V4049": [268, 269],
    "V4050": [269, 270],
    "V40501": [270, 271],
    "V405011": [271, 272],
    "V405012": [272, 280],
    "V40502": [280, 281],
    "V405021": [281, 282],
    "V405022": [282, 290],
    "V40503": [290, 291],
    "V405031": [291, 292],
    "V4051": [292, 293],
    "V40511": [293, 294],
    "V405111": [294, 295],
    "V405112": [295, 303],
    "V40512": [303, 304],
    "V405121": [304, 305],
    "V405122": [305, 313],
    "V4056": [313, 316],
    "V4056C": [316, 319],
    "V4057": [319, 320],
    "V4058": [320, 321],
    "V40581": [321, 322],
    "V405811": [322, 323],
    "V405812": [323, 331],
    "V40582": [331, 332],
    "V405821": [332, 333],
    "V405822": [333, 341],
    "V40583": [341, 342],
    "V405831": [342, 343],
    "V40584": [343, 344],
    "V4059": [344, 345],
    "V40591": [345, 346],
    "V405911": [346, 347],
    "V405912": [347, 355],
    "V40592": [355, 356],
    "V405921": [356, 357],
    "V405922": [357, 365],
    "V4062": [365, 368],
    "V4062C": [368, 371],
    "V4063": [371, 372],
    "V4063A": [372, 373],
    "V4064": [373, 374],
    "V4064A": [374, 375],
    "V4071": [375, 376],
    "V4072": [376, 378],
    "V4072A": [378, 379],
    "V4073": [379, 380],
    "V4074": [380, 381],
    "V4074A": [381, 383],
    "V4075A": [383, 384],
    "V4075A1": [384, 386],
    "V4076": [386, 387],
    "V40761": [387, 389],
    "V40762": [389, 391],
    "V40763": [391, 393],
    "V4077": [393, 394],
    "V4078": [394, 395],
    "V4078A": [395, 396],
    "V4082": [396, 397],
    "VD2002": [397, 399],
    "VD2003": [399, 401],
    "VD2004": [401, 402],
    "VD2006": [402, 404],
    "VD3004": [404, 405],
    "VD3005": [405, 407],
    "VD3006": [407, 408],
    "VD4001": [408, 409],
    "VD4002": [409, 410],
    "VD4003": [410, 411],
    "VD4004": [411, 412],
    "VD4004A": [412, 413],
    "VD4005": [413, 414],
    "VD4007": [414, 415],
    "VD4008": [415, 416],
    "VD4009": [416, 418],
    "VD4010": [418, 420],
    "VD4011": [420, 422],
    "VD4012": [422, 423],
    "VD4013": [423, 424],
    "VD4014": [424, 425],
    "VD4015": [425, 426],
    "VD4016": [426, 434],
    "VD4017": [434, 442],
    "VD4018": [442, 443],
    "VD4019": [443, 451],
    "VD4020": [451, 459],
    "VD4023": [459, 460],
    "VD4030": [460, 461],
    "VD4031": [461, 464],
    "VD4032": [464, 467],
    "VD4033": [467, 470],
    "VD4034": [470, 473],
    "VD4035": [473, 476],
    "VD4036": [476, 477],
    "VD4037": [477, 478]
}

VARIABLES_MAPPING = {
    "UF": {
        11: "Rondônia",
        12: "Acre",
        13: "Amazonas",
        14: "Roraima",
        15: "Pará",
        16: "Amapá",
        17: "Tocantins",
        21: "Maranhão",
        22: "Piauí",
        23: "Ceará",
        24: "Rio Grande do Norte",
        25: "Paraíba",
        26: "Pernambuco",
        27: "Alagoas",
        28: "Sergipe",
        29: "Bahia",
        31: "Minas Gerais",
        32: "Espírito Santo",
        33: "Rio de Janeiro",
        35: "São Paulo",
        41: "Paraná",
        42: "Santa Catarina",
        43: "Rio Grande do Sul",
        50: "Mato Grosso do Sul",
        51: "Mato Grosso",
        52: "Goiás",
        53: "Distrito Federal"
    },
    "Capital": {
        11: "Município de Porto Velho (RO)",
        12: "Município de Rio Branco (AC)",
        13: "Município de Manaus (AM)",
        14: "Município de Boa Vista (RR)",
        15: "Município de Belém (PA)",
        16: "Município de Macapá (AP)",
        17: "Município de Palmas (TO)",
        21: "Município de São Luís (MA)",
        22: "Município de Teresina (PI)",
        23: "Município de Fortaleza (CE)",
        24: "Município de Natal (RN)",
        25: "Município de João Pessoa (PB)",
        26: "Município de Recife (PE)",
        27: "Município de Maceió (AL)",
        28: "Município de Aracaju (SE)",
        29: "Município de Salvador (BA)",
        31: "Município de Belo Horizonte (MG)",
        32: "Município de Vitória (ES)",
        33: "Município de Rio de Janeiro (RJ)",
        35: "Município de São Paulo (SP)",
        41: "Município de Curitiba (PR)",
        42: "Município de Florianópolis (SC)",
        43: "Município de Porto Alegre (RS)",
        50: "Município de Campo Grande (MS)",
        51: "Município de Cuiabá (MT)",
        52: "Município de Goiânia (GO)",
        53: "Município de Brasília (DF)"
    },
    "RM_RIDE": {
        13: "Região Metropolitana de Manaus (AM)",
        15: "Região Metropolitana de Belém (PA)",
        16: "Região Metropolitana de Macapá (AP)",
        21: "Região Metropolitana de Grande São Luís (MA)",
        22: "Região Administrativa Integrada de Desenvolvimento da Grande Teresina (PI)",
        23: "Região Metropolitana de Fortaleza (CE)",
        24: "Região Metropolitana de Natal (RN)",
        25: "Região Metropolitana de João Pessoa (PB)",
        26: "Região Metropolitana de Recife (PE)",
        27: "Região Metropolitana de Maceió (AL)",
        28: "Região Metropolitana de Aracaju (SE)",
        29: "Região Metropolitana de Salvador (BA)",
        31: "Região Metropolitana de Belo Horizonte (MG)",
        32: "Região Metropolitana de Grande Vitória (ES)",
        33: "Região Metropolitana de Rio de Janeiro (RJ)",
        35: "Região Metropolitana de São Paulo (SP)",
        41: "Região Metropolitana de Curitiba (PR)",
        42: "Região Metropolitana de Florianópolis (SC)",
        43: "Região Metropolitana de Porto Alegre (RS)",
        51: "Região Metropolitana de Vale do Rio Cuiabá (MT)",
        52: "Região Metropolitana de Goiânia (GO)"
    },
    "V1022": {
        1: "Urbana",
        2: "Rural"
    },
    "V1023": {
        1: "Capital",
        2: "Resto da RM (Região Metropolitana, excluindo a capital)",
        3: "Resto da RIDE (Região Integrada de Desenvolvimento Econômico, excluindo a capital)",
        4: "Resto da UF  (Unidade da Federação, excluindo a região metropolitana e a RIDE)"
    },
    "V2005": {
        1: "Pessoa responsável pelo domicílio",
        2: "Cônjuge ou companheiro(a) de sexo diferente",
        3: "Cônjuge ou companheiro(a) do mesmo sexo",
        4: "Filho(a) do responsável e do cônjuge",
        5: "Filho(a) somente do responsável",
        6: "Enteado(a)",
        7: "Genro ou nora",
        8: "Pai, mãe, padrasto ou madrasta",
        9: "Sogro(a)",
        10: "Neto(a)",
        11: "Bisneto(a)",
        12: "Irmão ou irmã",
        13: "Avô ou avó",
        14: "Outro parente",
        15: "Agregado(a) - Não parente que não compartilha despesas",
        16: "Convivente - Não parente que compartilha despesas",
        17: "Pensionista",
        18: "Empregado(a) doméstico(a)",
        19: "Parente do(a) empregado(a) doméstico(a)"
    },
    "V2007": {
        1: "Homem",
        2: "Mulher"
    },
    "V2010": {
        1: "Branca",
        2: "Preta",
        3: "Amarela",
        4: "Parda",
        5: "Indígena",
        9: "Ignorado"
    },
    "V3001": {
        1: "Sim",
        2: "Não"
    },
    "V3002": {
        1: "Sim",
        2: "Não"
    },
    "V3002A": {
        1: "Rede privada",
        2: "Rede pública"
    },
    "V3003": {
        1: "Pré-escolar (maternal e jardim de infância)",
        2: "Alfabetização de jovens e adultos",
        3: "Regular do ensino fundamental",
        4: "Educação de jovens e adultos (EJA) ou supletivo do ensino fundamental",
        5: "Regular do ensino médio",
        6: "Educação de jovens e adultos (EJA) ou supletivo do ensino médio",
        7: "Superior - graduação",
        8: "Mestrado",
        9: "Doutorado"
    },
    "V3003A": {
        1: "Creche (disponível apenas no questionário anual de educação)",
        2: "Pré-escola",
        3: "Alfabetização de jovens e adultos",
        4: "Regular do ensino fundamental",
        5: "Educação de jovens e adultos (EJA) do ensino fundamental",
        6: "Regular do ensino médio",
        7: "Educação de jovens e adultos (EJA) do ensino médio",
        8: "Superior - graduação",
        9: "Especialização de nível superior",
        10: "Mestrado",
        11: "Doutorado"
    },
    "V3004": {
        1: "8 anos",
        2: "9 anos"
    },
    "V3005": {
        1: "Sim",
        2: "Não"
    },
    "V3005A": {
        1: "Períodos semestrais",
        2: "Anos",
        3: "Outra forma"
    },
    "V3006": {
        1: "Primeira (o)",
        2: "Segunda (o)",
        3: "Terceira (o)",
        4: "Quarta (o)",
        5: "Quinta (o)",
        6: "Sexta (o)",
        7: "Sétima (o)",
        8: "Oitava (o)",
        9: "Nona (o)",
        10: "Décimo",
        11: "Décimo primeiro",
        12: "Décimo segundo",
        13: "Curso não classificado em séries ou anos"
    },
    "V3006A": {
        1: "Anos iniciais (primeiro segmento)",
        2: "Anos finais (segundo segmento)"
    },
    "V3007": {
        1: "Sim",
        2: "Não"
    },
    "V3008": {
        1: "Sim",
        2: "Não"
    },
    "V3009": {
        1: "Classe de alfabetização - CA",
        2: "Alfabetização de jovens e adultos",
        3: "Antigo primário (elementar)",
        4: "Antigo ginásio (médio 1º ciclo)",
        5: "Regular do ensino fundamental ou do 1º grau",
        6: "Educação de jovens e adultos (EJA) ou supletivo do ensino fundamental",
        7: "Antigo científico, clássico, etc. (médio 2º ciclo)",
        8: "Regular do ensino médio ou do 2º grau",
        9: "Educação de jovens e adultos (EJA) ou supletivo do ensino médio",
        10: "Superior - graduação",
        11: "Mestrado",
        12: "Doutorado"
    },
    "V3009A": {
        1: "Creche (disponível apenas no questionário anual de educação)",
        2: "Pré-escola",
        3: "Classe de alfabetização - CA",
        4: "Alfabetização de jovens e adultos",
        5: "Antigo primário (elementar)",
        6: "Antigo ginásio (médio 1º ciclo)",
        7: "Regular do ensino fundamental ou do 1º grau",
        8: "Educação de jovens e adultos (EJA) ou supletivo do 1º grau",
        9: "Antigo científico, clássico, etc. (médio 2º ciclo)",
        10: "Regular do ensino médio ou do 2º grau",
        11: "Educação de jovens e adultos (EJA) ou supletivo do 2º grau",
        12: "Superior - graduação",
        13: "Especialização de nível superior",
        14: "Mestrado",
        15: "Doutorado"
    },
    "V3010": {
        1: "8 anos",
        2: "9 anos"
    },
    "V3011": {
        1: "Sim",
        2: "Não"
    },
    "V3011A": {
        1: "Períodos semestrais",
        2: "Anos",
        3: "Outra forma"
    },
    "V3012": {
        1: "Concluiu",
        2: "Não concluiu",
        3: "Curso não classificado em séries ou anos"
    },
    "V3013": {
        1: "Primeira (o)",
        2: "Segunda (o)",
        3: "Terceira (o)",
        4: "Quarta (o)",
        5: "Quinta (o)",
        6: "Sexta (o)",
        7: "Sétima (o)",
        8: "Oitava (o)",
        9: "Nona (o)",
        10: "Décimo",
        11: "Décimo primeiro",
        12: "Décimo segundo"
    },
    "V3013A": {
        1: "Anos iniciais (primeiro segmento)",
        2: "Anos finais (segundo segmento)"
    },
    "V3013B": {
        1: "Sim",
        2: "Não"
    },
    "V3014": {
        1: "Sim",
        2: "Não"
    },
    "V4001": {
        1: "Sim",
        2: "Não"
    },
    "V4002": {
        1: "Sim",
        2: "Não"
    },
    "V4003": {
        1: "Sim",
        2: "Não"
    },
    "V4004": {
        1: "Sim",
        2: "Não"
    },
    "V4005": {
        1: "Sim",
        2: "Não"
    },
    "V4006": {
        1: "Férias, folga ou jornada de trabalho variável",
        2: "Licença maternidade",
        3: "Licença remunerada por motivo de doença ou acidente da própria pessoa",
        4: "Outro tipo de licença remunerada (estudo, paternidade, casamento, licença prêmio, etc.)",
        5: "Afastamento do próprio negócio/empresa por motivo de gestação, doença, acidente, etc., sem ser remunerado por instituto de previdência",
        6: "Fatores ocasionais (tempo, paralisação nos serviços de transportes, etc.)",
        7: "Greve ou paralisação",
        8: "Outro motivo"
    },
    "V4006A": {
        1: "Férias, folga ou jornada de trabalho variável",
        2: "Licença maternidade ou paternidade",
        3: "Licença remunerada por motivo de saúde ou acidente da própria pessoa",
        4: "Outro tipo de licença remunerada (estudo, casamento, licença prêmio, etc.)",
        5: "Afastamento do próprio negócio/empresa por motivo de gestação, doença, acidente, etc., sem ser remunerado por instituto de previdência",
        6: "Fatores ocasionais (tempo, paralisação nos serviços de transportes, etc.)",
        7: "Outro motivo"
    },
    "V4007": {
        1: "Sim",
        2: "Não"
    },
    "V4008": {
        1: "Menos de 1 mês",
        2: "De 1 mês a menos de 1 ano",
        3: "De 1 ano a menos de 2 anos",
        4: "2 anos ou mais"
    },
    "V4009": {
        1: "Um",
        2: "Dois",
        3: "Três ou mais"
    },
    "V4010": CNAE_OCUPATION,
    "V4012": {
        1: "Trabalhador doméstico",
        2: "Militar do exército, da marinha, da aeronáutica, da polícia militar ou do corpo de bombeiros militar",
        3: "Empregado do setor privado",
        4: "Empregado do setor público (inclusive empresas de economia mista)",
        5: "Empregador",
        6: "Conta própria",
        7: "Trabalhador familiar não remunerado"
    },
    "V40121": {
        1: "Em ajuda a conta própria ou empregador",
        2: "Em ajuda a empregado",
        3: "Em ajuda a trabalhador doméstico"
    },
    "V4013": CNAE_ACTIVITY,
    "V40132": {
        1: "Agricultura, pecuária silvicultura, exploração florestal, pesca ou aquicultura",
        2: "Outra atividade, inclusive as atividades de apoio à agricultura, pecuária, silvicultura, exploração florestal, pesca ou aquicultura."
    },
    "V40132A": {
        1: "Agricultura, pecuária silvicultura, exploração florestal, pesca ou aquicultura e atividades de apoio à agricultura, pecuária, silvicultura, exploração florestal, pesca ou aquicultura.",
        2: "Outra atividade"
    },
    "V4014": {
        1: "Federal",
        2: "Estadual",
        3: "Municipal"
    },
    "V4015": {
        1: "Sim",
        2: "Não"
    },
    "V40151": {
        1: "1 a 5 trabalhadores não remunerados",
        2: "6 a 10 trabalhadores não remunerados",
        3: "11 ou mais trabalhadores não remunerados"
    },
    "V4016": {
        1: "1 a 5 empregados",
        2: "6 a 10 empregados",
        3: "11 a 50 empregados",
        4: "51 ou mais empregados"
    },
    "V4017": {
        1: "Sim",
        2: "Não"
    },
    "V40171": {
        1: "1 a 5 sócios",
        2: "6 ou mais sócios"
    },
    "V4018": {
        1: "1 a 5 pessoas",
        2: "6 a 10 pessoas",
        3: "11 a 50 pessoas",
        4: "51 ou mais pessoas"
    },
    "V4019": {
        1: "Sim",
        2: "Não"
    },
    "V4020": {
        1: "Em loja, escritório, galpão, etc.",
        2: "Em fazenda, sítio, granja, chácara, etc.",
        3: "Não tinha estabelecimento para funcionar"
    },
    "V4021": {
        1: "Sim",
        2: "Não"
    },
    "V4022": {
        1: "Em estabelecimento de outro négocio/empresa",
        2: "Em local designado pelo empregador, cliente ou freguês",
        3: "Em domicílio de empregador, patrão, sócio ou freguês",
        4: "No domicílio de residência, em local exclusivo para o desempenho da atividade",
        5: "No domicílio de residência, sem local exclusivo para o desempenho da atividade",
        6: "Em veículo automotor (táxi, ônibus, caminhão, automóvel, embarcação, etc.)",
        7: "Em via ou área pública (rua, rio, manguezal, mata pública, praça, praia etc.)",
        8: "Em outro local, especifique"
    },
    "V4024": {
        1: "Sim",
        2: "Não"
    },
    "V4025": {
        1: "Sim",
        2: "Não"
    },
    "V4026": {
        1: "Sim",
        2: "Não"
    },
    "V4027": {
        1: "Sim",
        2: "Não"
    },
    "V4028": {
        1: "Sim",
        2: "Não"
    },
    "V4029": {
        1: "Sim",
        2: "Não"
    },
    "V4032": {
        1: "Sim",
        2: "Não"
    },
    "V4033": {
        1: "O quesito foi respondido"
    },
    "V40331": {
        1: "Em dinheiro"
    },
    "V403311": {
        1: "1 a [0,5SM]",
        2: "[0,5SM]+1 a [1SM]",
        3: "[1SM]+1 a [2SM]",
        4: "[2SM]+1 a [3SM]",
        5: "[3SM]+1 a [5SM]",
        6: "[5SM]+1 a [10SM]",
        7: "[10SM]+1 a [20SM]",
        8: "[20SM]+1 ou mais"
    },
    "V40332": {
        2: "Em produtos ou mercadorias"
    },
    "V403321": {
        1: "1 a [0,5SM]",
        2: "[0,5SM]+1 a [1SM]",
        3: "[1SM]+1 a [2SM]",
        4: "[2SM]+1 a [3SM]",
        5: "[3SM]+1 a [5SM]",
        6: "[5SM]+1 a [10SM]",
        7: "[10SM]+1 a [20SM]",
        8: "[20SM]+1 ou mais"
    },
    "V40333": {
        3: "Em benefícios"
    },
    "V403331": {
        1: "Pessoa recebendo somente em beneficios, exceto aprendizado",
        2: "Aprendiz ou estagiário recebendo em aprendizado e outros beneficios",
        3: "Aprendiz ou estagiário recebendo somente em aprendizado"
    },
    "V4034": {
        1: "O quesito foi respondido"
    },
    "V40341": {
        1: "Em dinheiro"
    },
    "V403411": {
        0: "0",
        1: "1 a [0,5SM]",
        2: "[0,5SM]+1 a [1SM]",
        3: "[1SM]+1 a [2SM]",
        4: "[2SM]+1 a [3SM]",
        5: "[3SM]+1 a [5SM]",
        6: "[5SM]+1 a [10SM]",
        7: "[10SM]+1 a [20SM]",
        8: "[20SM]+1 ou mais"
    },
    "V40342": {
        2: "Em produtos ou mercadorias"
    },
    "V403421": {
        0: "0",
        1: "1 a [0,5SM]",
        2: "[0,5SM]+1 a [1SM]",
        3: "[1SM]+1 a [2SM]",
        4: "[2SM]+1 a [3SM]",
        5: "[3SM]+1 a [5SM]",
        6: "[5SM]+1 a [10SM]",
        7: "[10SM]+1 a [20SM]",
        8: "[20SM]+1 ou mais"
    },
    "V4040": {
        1: "Menos de 1 mês",
        2: "De 1 mês a menos de 1 ano",
        3: "De 1 ano a menos de 2 anos",
        4: "2 anos ou mais"
    },
    "V4041": CNAE_OCUPATION,
    "V4043": {
        1: "Trabalhador doméstico",
        2: "Militar do exército, da marinha, da aeronáutica, da polícia militar ou do corpo de bombeiros militar",
        3: "Empregado do setor privado",
        4: "Empregado do setor público (inclusive empresas de economia mista)",
        5: "Empregador",
        6: "Conta própria",
        7: "Trabalhador não remunerado em ajuda a membro do domicílio ou parente"
    },
    "V40431": {
        1: "Em ajuda a conta própria ou empregador",
        2: "Em ajuda a empregado",
        3: "Em ajuda a trabalhador doméstico"
    },
    "V4044": CNAE_ACTIVITY,
    "V4045": {
        1: "Federal",
        2: "Estadual",
        3: "Municipal"
    },
    "V4046": {
        1: "Sim",
        2: "Não"
    },
    "V4047": {
        1: "Sim",
        2: "Não"
    },
    "V4048": {
        1: "Sim",
        2: "Não"
    },
    "V4049": {
        1: "Sim",
        2: "Não"
    },
    "V4050": {
        1: "O quesito foi respondido"
    },
    "V40501": {
        1: "Em dinheiro"
    },
    "V405011": {
        1: "1 a [0,5SM]",
        2: "[0,5SM]+1 a [1SM]",
        3: "[1SM]+1 a [2SM]",
        4: "[2SM]+1 a [3SM]",
        5: "[3SM]+1 a [5SM]",
        6: "[5SM]+1 a [10SM]",
        7: "[10SM]+1 a [20SM]",
        8: "[20SM]+1 ou mais"
    },
    "V40502": {
        2: "Em produtos ou mercadorias"
    },
    "V405021": {
        1: "1 a [0,5SM]",
        2: "[0,5SM]+1 a [1SM]",
        3: "[1SM]+1 a [2SM]",
        4: "[2SM]+1 a [3SM]",
        5: "[3SM]+1 a [5SM]",
        6: "[5SM]+1 a [10SM]",
        7: "[10SM]+1 a [20SM]",
        8: "[20SM]+1 ou mais"
    },
    "V40503": {
        3: "Em benefícios"
    },
    "V405031": {
        1: "Pessoa recebendo somente em beneficios, exceto aprendizado",
        2: "Aprendiz ou estagiário recebendo em aprendizado e outros beneficios",
        3: "Aprendiz ou estagiário recebendo somente em aprendizado"
    },
    "V4051": {
        1: "O quesito foi respondido"
    },
    "V40511": {
        1: "Em dinheiro"
    },
    "V405111": {
        0: "0",
        1: "1 a [0,5SM]",
        2: "[0,5SM]+1 a [1SM]",
        3: "[1SM]+1 a [2SM]",
        4: "[2SM]+1 a [3SM]",
        5: "[3SM]+1 a [5SM]",
        6: "[5SM]+1 a [10SM]",
        7: "[10SM]+1 a [20SM]",
        8: "[20SM]+1 ou mais"
    },
    "V40512": {
        2: "Produtos ou mercadorias"
    },
    "V405121": {
        0: "0",
        1: "1 a [0,5SM]",
        2: "[0,5SM]+1 a [1SM]",
        3: "[1SM]+1 a [2SM]",
        4: "[2SM]+1 a [3SM]",
        5: "[3SM]+1 a [5SM]",
        6: "[5SM]+1 a [10SM]",
        7: "[10SM]+1 a [20SM]",
        8: "[20SM]+1 ou mais"
    },
    "V4057": {
        1: "Sim",
        2: "Não"
    },
    "V4058": {
        1: "O quesito foi respondido"
    },
    "V40581": {
        1: "Em dinheiro"
    },
    "V405811": {
        1: "1 a [0,5SM]",
        2: "[0,5SM]+1 a [1SM]",
        3: "[1SM]+1 a [2SM]",
        4: "[2SM]+1 a [3SM]",
        5: "[3SM]+1 a [5SM]",
        6: "[5SM]+1 a [10SM]",
        7: "[10SM]+1 a [20SM]",
        8: "[20SM]+1 ou mais"
    },
    "V40582": {
        2: "Em produtos ou mercadorias"
    },
    "V405821": {
        1: "1 a [0,5SM]",
        2: "[0,5SM]+1 a [1SM]",
        3: "[1SM]+1 a [2SM]",
        4: "[2SM]+1 a [3SM]",
        5: "[3SM]+1 a [5SM]",
        6: "[5SM]+1 a [10SM]",
        7: "[10SM]+1 a [20SM]",
        8: "[20SM]+1 ou mais"
    },
    "V40583": {
        3: "Em benefícios"
    },
    "V405831": {
        1: "Pessoa recebendo somente em beneficios, exceto aprendizado",
        2: "Aprendiz ou estagiário recebendo em aprendizado e outros beneficios",
        3: "Aprendiz ou estagiário recebendo somente em aprendizado"
    },
    "V40584": {
        4: "Não remunerado"
    },
    "V4059": {
        1: "O quesito foi respondido"
    },
    "V40591": {
        1: "Em dinheiro"
    },
    "V405911": {
        0: "0",
        1: "1 a [0,5SM]",
        2: "[0,5SM]+1 a [1SM]",
        3: "[1SM]+1 a [2SM]",
        4: "[2SM]+1 a [3SM]",
        5: "[3SM]+1 a [5SM]",
        6: "[5SM]+1 a [10SM]",
        7: "[10SM]+1 a [20SM]",
        8: "[20SM]+1 ou mais"
    },
    "V40592": {
        2: "Em produtos ou mercadorias"
    },
    "V405921": {
        0: "0",
        1: "1 a [0,5SM]",
        2: "[0,5SM]+1 a [1SM]",
        3: "[1SM]+1 a [2SM]",
        4: "[2SM]+1 a [3SM]",
        5: "[3SM]+1 a [5SM]",
        6: "[5SM]+1 a [10SM]",
        7: "[10SM]+1 a [20SM]",
        8: "[20SM]+1 ou mais"
    },
    "V4063": {
        1: "Sim",
        2: "Não"
    },
    "V4063A": {
        1: "Sim",
        2: "Não"
    },
    "V4064": {
        1: "Sim",
        2: "Não"
    },
    "V4064A": {
        1: "Sim",
        2: "Não"
    },
    "V4071": {
        1: "Sim",
        2: "Não"
    },
    "V4072": {
        1: "Entrou diretamente em contato com empregador (em fábrica, fazenda, mercado, loja ou outro local de trabalho)",
        2: "Fez ou inscreveu-se em concurso",
        3: "Consultou agência privada ou sindicato",
        4: "Consultou agência municipal, estadual ou o Sistema Nacional de Emprego (SINE)",
        5: "Colocou ou respondeu anúncio",
        6: "Consultou parente, amigo ou colega",
        7: "Buscou ajuda financeira para iniciar o próprio negócio",
        8: "Procurou local, equipamento ou maquinário para iniciar o próprio negócio",
        9: "Solicitou registro ou licença para iniciar o próprio negócio",
        10: "Tomou outra providência",
        11: "Não tomou providência efetiva"
    },
    "V4072A": {
        1: "Entrou em contato com empregador (pessoalmente, por telefone, por email ou pelo portal da empresa, inclusive enviando currículo)",
        2: "Colocou ou respondeu anúncio de trabalho em jornal ou revista",
        3: "Consultou ou inscreveu-se em agência de emprego privada ou sindicato",
        4: "Consultou ou inscreveu-se em agência de emprego municipal, estadual ou no Sistema Nacional de Emprego (SINE)",
        5: "Fez ou inscreveu-se em concurso",
        6: "Consultou parente, amigo ou colega",
        7: "Tomou medida para iniciar o próprio negócio (recursos financeiros, local para instalação, equipamentos, legalização etc.)",
        8: "Tomou outra providência, especifique:",
        9: "Não tomou providência efetiva"
    },
    "V4073": {
        1: "Sim",
        2: "Não"
    },
    "V4074": {
        1: "Conseguiu proposta de trabalho para começar após a semana de referência",
        2: "Aguardando resposta de medida tomada para conseguir trabalho",
        3: "Desistiu de procurar por não conseguir encontrar trabalho",
        4: "Acha que não vai encontrar trabalho por ser muito jovem ou muito idoso",
        5: "Tinha que cuidar de filho(s), de outro(s) dependente(s) ou dos afazeres domésticos",
        6: "Estudo",
        7: "Incapacidade física, mental ou doença permanente",
        8: "Outro motivo"
    },
    "V4074A": {
        1: "Conseguiu proposta de trabalho para começar após a semana de referência",
        2: "Estava aguardando resposta de medida tomada para conseguir trabalho",
        3: "Não conseguia trabalho adequado",
        4: "Não tinha experiência profissional ou qualificação",
        5: "Não conseguia trabalho por ser considerado muito jovem ou muito idoso",
        6: "Não havia trabalho na localidade",
        7: "Tinha que cuidar dos afazeres domésticos, do(s) filho(s) ou de outro(s) parente(s)",
        8: "Estava estudando (curso de qualquer tipo ou por conta própria)",
        9: "Por problema de saúde ou gravidez",
        10: "Outro motivo, especifique"
    },
    "V4075A": {
        1: "Menos de 1 mês",
        2: "De 1 mês a menos de 1 ano",
        3: "1 ano ou mais"
    },
    "V4076": {
        1: "Menos de 1 mês",
        2: "De 1 mês a menos de 1 ano",
        3: "De 1 ano a menos de 2 anos",
        4: "2 anos ou mais"
    },
    "V4077": {
        1: "Sim",
        2: "Não"
    },
    "V4078": {
        1: "Tinha que cuidar de filho(s), de outro(s) dependente(s) ou dos afazeres domésticos",
        2: "Estudo",
        3: "Incapacidade física, mental ou doença permanente",
        4: "Aposentado ou idoso para trabalhar",
        5: "Muito jovem para trabalhar",
        6: "Não desejava trabalhar",
        7: "Outro motivo"
    },
    "V4078A": {
        1: "Tinha que cuidar dos afazeres domésticos, do(s) filho(s) ou de outro(s) parente(s)",
        2: "Estava estudando (em curso de qualquer tipo ou por conta própria)",
        3: "Por problema de saúde ou gravidez",
        4: "Por ser muito jovem ou muito idoso para trabalhar",
        5: "Por não querer trabalhar",
        6: "Outro motivo, especifique:"
    },
    "V4082": {
        1: "Sim",
        2: "Não"
    },
    "VD2002": {
        1: "Pessoa responsável",
        2: "Cônjuge ou companheiro(a)",
        3: "Filho(a)",
        4: "Enteado(a)",
        5: "Genro ou nora",
        6: "Pai, mãe, padrasto ou madrasta",
        7: "Sogro(a)",
        8: "Neto(a)",
        9: "Bisneto(a)",
        10: "Irmão ou irmã",
        11: "Avô ou avó",
        12: "Outro parente",
        13: "Agregado(a)",
        14: "Convivente",
        15: "Pensionista",
        16: "Empregado(a) doméstico(a)",
        17: "Parente do(a) empregado(a) doméstico(a)"
    },
    "VD2004": {
        1: "Unipessoal",
        2: "Nuclear",
        3: "Estendida",
        4: "Composta"
    },
    "VD2006": {
        1: "0 a 4 anos",
        2: "5 a 9 anos",
        3: "10 a 13 anos",
        4: "14 a 19 anos",
        5: "20 a 24 anos",
        6: "25 a 29 anos",
        7: "30 a 34 anos",
        8: "35 a 39 anos",
        9: "40 a 44 anos",
        10: "45 a 49 anos",
        11: "50 a 54 anos",
        12: "55 a 59 anos",
        13: "60 a 64 anos",
        14: "65 a 69 anos",
        15: "70 a 74 anos",
        16: "75 a 79 anos",
        17: "80 anos ou mais"
    },
    "VD3004": {
        1: "Sem instrução e menos de 1 ano de estudo",
        2: "Fundamental incompleto ou equivalente",
        3: "Fundamental completo ou equivalente",
        4: "Médio incompleto ou equivalente",
        5: "Médio completo ou equivalente",
        6: "Superior incompleto ou equivalente",
        7: "Superior completo"
    },
    "VD3005": {
        0: "Sem instrução e menos de 1 ano de estudo",
        1: "1 ano de estudo",
        2: "2 anos de estudo",
        3: "3 anos de estudo",
        4: "4 anos de estudo",
        5: "5 anos de estudo",
        6: "6 anos de estudo",
        7: "7 anos de estudo",
        8: "8 anos de estudo",
        9: "9 anos de estudo",
        10: "10 anos de estudo",
        11: "11 anos de estudo",
        12: "12 anos de estudo",
        13: "13 anos de estudo",
        14: "14 anos de estudo",
        15: "15 anos de estudo",
        16: "16 anos ou mais de estudo"
    },
    "VD3006": {
        1: "Sem instrução e menos de 1 ano de estudo",
        2: "1 a 4 anos de estudo",
        3: "5 a 8 anos de estudo",
        4: "9 a 11 anos de estudo",
        5: "12 a 15 anos de estudo",
        6: "16 anos ou mais de estudo"
    },
    "VD4001": {
        1: "Pessoas na força de trabalho",
        2: "Pessoas fora da força de trabalho"
    },
    "VD4002": {
        1: "Pessoas ocupadas",
        2: "Pessoas desocupadas"
    },
    "VD4003": {
        1: "Pessoas fora da força de trabalho e na força de trabalho potencial",
        2: "Pessoas fora da força de trabalho e fora da força de trabalho potencial"
    },
    "VD4004": {
        1: "Pessoas subocupadas"
    },
    "VD4004A": {
        1: "Pessoas subocupadas"
    },
    "VD4005": {
        1: "Pessoas desalentadas"
    },
    "VD4007": {
        1: "Empregado (inclusive trabalhador doméstico)",
        2: "Empregador",
        3: "Conta própria",
        4: "Trabalhador familiar auxiliar"
    },
    "VD4008": {
        1: "Empregado no setor privado",
        2: "Trabalhador doméstico",
        3: "Empregado no setor público (inclusive servidor estatutário e militar)",
        4: "Empregador",
        5: "Conta-própria",
        6: "Trabalhador familiar auxiliar"
    },
    "VD4009": {
        1: "Empregado no setor privado com carteira de trabalho assinada",
        2: "Empregado no setor privado sem carteira de trabalho assinada",
        3: "Trabalhador doméstico com carteira de trabalho assinada",
        4: "Trabalhador doméstico sem carteira de trabalho assinada",
        5: "Empregado no setor público com carteira de trabalho assinada",
        6: "Empregado no setor público sem carteira de trabalho assinada",
        7: "Militar e servidor estatutário",
        8: "Empregador",
        9: "Conta-própria",
        10: "Trabalhador familiar auxiliar"
    },
    "VD4010": {
        1: "Agricultura, pecuária, produção florestal, pesca e aquicultura",
        2: "Indústria geral",
        3: "Construção",
        4: "Comércio, reparação de veículos automotores e motocicletas",
        5: "Transporte, armazenagem e correio",
        6: "Alojamento e alimentação",
        7: "Informação, comunicação e atividades financeiras, imobiliárias, profissionais e administrativas",
        8: "Administração pública, defesa e seguridade social",
        9: "Educação, saúde humana e serviços sociais",
        10: "Outros Serviços",
        11: "Serviços domésticos",
        12: "Atividades mal definidas"
    },
    "VD4011": {
        1: "Diretores e gerentes",
        2: "Profissionais das ciências e intelectuais",
        3: "Técnicos e profissionais de nível médio",
        4: "Trabalhadores de apoio administrativo",
        5: "Trabalhadores dos serviços, vendedores dos comércios e mercados",
        6: "Trabalhadores qualificados da agropecuária, florestais, da caça e da pesca",
        7: "Trabalhadores qualificados, operários e artesões da construção, das artes mecânicas e outros ofícios",
        8: "Operadores de instalações e máquinas e montadores",
        9: "Ocupações elementares",
        10: "Membros das forças armadas, policiais e bombeiros militares",
        11: "Ocupações maldefinidas"
    },
    "VD4012": {
        1: "Contribuinte",
        2: "Não contribuinte"
    },
    "VD4013": {
        1: "Até 14 horas",
        2: "15 a 39 horas",
        3: "40 a 44 horas",
        4: "45 a 48 horas",
        5: "49 horas ou mais"
    },
    "VD4014": {
        1: "Até 14 horas",
        2: "15 a 39 horas",
        3: "40 a 44 horas",
        4: "45 a 48 horas",
        5: "49 horas ou mais"
    },
    "VD4015": {
        1: "Remuneração em dinheiro, produtos ou mercadorias no trabalho principal",
        2: "Remuneração em benefícios ou sem remuneração no trabalho principal"
    },
    "VD4018": {
        1: "Remuneração em dinheiro, produtos ou mercadorias em pelo menos um dos trabalhos",
        2: "Remuneração em benefícios ou sem remuneração em todos os trabalhos"
    },
    "VD4023": {
        1: "Tinha que cuidar dos afazeres domésticos, do(s) filho(s) ou de outro(s) dependente(s)",
        2: "Estava estudando",
        3: "Por incapacidade física, mental ou doença permanente",
        4: "Por ser muito jovem ou muito idoso para trabalhar",
        5: "Por não querer trabalhar",
        6: "Por outro motivo"
    },
    "VD4030": {
        1: "Tinha que cuidar dos afazeres domésticos, do(s) filho(s) ou de outro(s) parente(s)",
        2: "Estava estudando",
        3: "Por problema de saúde ou gravidez",
        4: "Por ser muito jovem ou muito idoso para trabalhar",
        5: "Por não querer trabalhar",
        6: "Por outro motivo"
    },
    "VD4036": {
        1: "Até 14 horas",
        2: "15 a 39 horas",
        3: "40 a 44 horas",
        4: "45 a 48 horas",
        5: "49 horas ou mais"
    },
    "VD4037": {
        1: "Até 14 horas",
        2: "15 a 39 horas",
        3: "40 a 44 horas",
        4: "45 a 48 horas",
        5: "49 horas ou mais"
    }
}

VARIABLES_DESCRIPTION = {
    "Ano": "Ano de referência",
    "Trimestre": "Trimestre de referência",
    "UF": "Unidade da Federação",
    "Capital": "Município da Capital",
    "RM_RIDE": "Região Metropolitana e Região Administrativa Integrada\nde Desenvolvimento",
    "V1022": "Situação do domicílio",
    "V1023": "Tipo de área",
    "V1028": "Peso do domicílio e das pessoas",
    "V2001": "Número de pessoas no domicílio",
    "V2005": "Condição no domicílio",
    "V2007": "Sexo",
    "V2008": "Dia de nascimento",
    "V20081": "Mês de nascimento",
    "V20082": "Ano de nascimento",
    "V2009": "Idade do morador na data de referência",
    "V2010": "Cor ou raça",
    "V3001": "... sabe ler e escrever?",
    "V3002": "... frequenta escola?",
    "V3002A": "A escola que ... frequenta é de",
    "V3003": "Qual é o curso que ... frequenta?",
    "V3003A": "Qual é o curso que ... frequenta?",
    "V3004": "A duração deste curso que ... frequenta é de:",
    "V3005": "Este curso que ... frequenta é seriado?",
    "V3005A": "Esse curso que .... frequenta é organizado em:",
    "V3006": "Qual é o ano/série/semestre que ... frequenta?",
    "V3006A": "Qual é a etapa do ensino fundamental que ... frequenta?",
    "V3007": "... já concluiu algum outro curso de graduação?",
    "V3008": "Anteriormente ... frequentou escola?",
    "V3009": "Qual foi o curso mais elevado que ... frequentou anteriormente?",
    "V3009A": "Qual foi o curso mais elevado que ... frequentou anteriormente?",
    "V3010": "A duração deste curso que ... frequentou anteriormente era de:",
    "V3011": "Este curso que ... frequentou anteriormente era seriado?",
    "V3011A": "Esse curso que .... frequentou era organizado em:",
    "V3012": "... concluiu com aprovação, pelo menos a primeira série deste curso que frequentou anteriormente?",
    "V3013": "Qual foi o último ano/série/semestre que ... concluiu com aprovação, neste curso que frequentou anteriormente",
    "V3013A": "Qual foi a etapa de ensino fundamental que ... frequentou?",
    "V3013B": "... Concluiu os anos iniciais deste curso que frequentou anteriormente?",
    "V3014": "... concluiu este curso que frequentou anteriormente",
    "V4001": "Na semana de ... a ... (semana de referência), ... trabalhou ou estagiou, durante pelo menos 1 hora, em alguma atividade remunerada em dinheiro?",
    "V4002": "Na semana de ... a ... (semana de referência), ... trabalhou ou estagiou, durante pelo menos 1 hora, em alguma atividade remunerada em produtos, mercadorias, moradia, alimentação, etc.?",
    "V4003": "Na semana de ... a ... (semana de referência), ... fez algum bico ou trabalhou em alguma atividade ocasional remunerada durante pelo menos 1 hora?",
    "V4004": "Na semana de ... a ... (semana de referência), ... ajudou durante pelo menos 1 hora, sem receber pagamento, no trabalho remunerado de algum morador do domicílio ou de parente?",
    "V4005": "Na semana de ... a ... (semana de referência), ... tinha algum trabalho remunerado do qual estava temporariamente afastado?",
    "V4006": "Na semana de ... a .... (semana de referência), por que motivo ... estava afastado desse trabalho?",
    "V4006A": "Na semana de ... a .... (semana de referência), por que motivo ... estava afastado desse trabalho?",
    "V4007": "Durante o tempo de afastamento, ... continuou a receber ao menos uma parte do pagamento?",
    "V4008": "Em .../.../...(último dia da semana de referência), fazia quanto tempo que ... estava afastado desse trabalho?",
    "V40081": "Tempo que estava afastado (De 1 mês a menos de 1 ano)",
    "V40082": "Tempo que estava afastado (De 1 ano a menos de 2 anos)",
    "V40083": "Tempo que estava afastado (de 02 anos a 98 anos)",
    "V4009": "Quantos trabalhos ... tinha na semana de ... a ... (semana de referência ?",
    "V4010": "Código da ocupação (cargo ou função)",
    "V4012": "Nesse trabalho, ... era:",
    "V40121": "Trabalhador não remunerado",
    "V4013": "Código da principal atividade desse negócio/empresa",
    "V40132": "Qual a seção da atividade?",
    "V40132A": "Qual a seção da atividade?",
    "V4014": "Esse trabalho era na área:",
    "V4015": "Na semana de ... a ... (semana de referência), ... teve ajuda, nesse trabalho, de pelo menos um trabalhador não remunerado que era membro do domicílio ou parente?",
    "V40151": "Quantos trabalhadores não remunerados ?",
    "V401511": "1 a 5 trabalhadores não remunerados",
    "V401512": "6 a 10 trabalhadores não remunerados",
    "V4016": "Na semana de ... a ... (semana de referência), quantos empregados trabalhavam nesse negócio/empresa que ... tinha ?",
    "V40161": "1 a 5 empregados",
    "V40162": "6 a 10 empregados",
    "V40163": "11 a 50 empregados",
    "V4017": "Na semana de ... a ... (semana de referência), ... tinha pelo menos um sócio que trabalhava nesse negócio/empresa ?",
    "V40171": "Quantos?",
    "V401711": "1 a 5 sócios",
    "V4018": "Na semana de ... a ... (semana de referência), contando com ... , quantas pessoas trabalhavam nesse negócio/empresa ?",
    "V40181": "1 a 5 pessoas",
    "V40182": "6 a 10 pessoas",
    "V40183": "11 a 50 pessoas",
    "V4019": "Esse negócio/empresa era registrado no Cadastro Nacional da Pessoa Jurídica - CNPJ?",
    "V4020": "Em que tipo de local funcionava esse negócio/empresa ?",
    "V4021": "... exercia normalmente o trabalho em estabelecimento desse negócio/empresa ?",
    "V4022": "Então onde ... exercia normalmente esse trabalho ?",
    "V4024": "No período de ... a ... (período de referência de 30 dias) ... prestava serviço doméstico em mais de um domicílio ?",
    "V4025": "Nesse trabalho, ... era contratado(a) como empregado temporário ?",
    "V4026": "Na semana de ... a ... (semana de referência), ... era contratado(a) somente por pessoa responsável pelo negócio/empresa em que trabalhava ?",
    "V4027": 'Na semana de ... a ... (semana de referência) era contratado(a) somente por intermediário (empresa empreiteira, empreiteiro, "gato", etc.) ?',
    "V4028": "Nesse trabalho, ... era servidor público estatutário (federal, estadual ou municipal) ?",
    "V4029": "Nesse trabalho, ... tinha carteira de trabalho assinada ?",
    "V4032": "Era contribuinte de instituto de previdência por esse trabalho ?",
    "V4033": "Qual era o rendimento bruto mensal que ... recebia/fazia normalmente nesse trabalho? (variável auxiliar)",
    "V40331": "Recebia/fazia normalmente nesse trabalho rendimento/retirada em dinheiro?",
    "V403311": "Número da faixa do rendimento/retirada em dinheiro",
    "V403312": "Qual era o rendimento bruto/retirada mensal que ... recebia/fazia normalmente nesse trabalho ? (valor em dinheiro)",
    "V40332": "Recebia/fazia normalmente nesse trabalho rendimento/retirada em produtos e mercadorias ?",
    "V403321": "Número da faixa do rendimento/retirada em produtos e mercadorias",
    "V403322": "Qual era o rendimento bruto/retirada mensal que ... recebia/fazia normalmente nesse trabalho ? (valor estimado dos produtos ou mercadorias)",
    "V40333": "Recebia/fazia normalmente nesse trabalho rendimento/retirada somente em benefícios ?",
    "V403331": "Tipo de rendimento/retirada em benefícios que recebia/fazia nesse trabalho",
    "V4034": "Qual foi o rendimento bruto que ... recebeu/fez nesse trabalho, no mês de referência? (variável auxiliar)",
    "V40341": "Recebeu/fez nesse trabalho rendimento/retirada em dinheiro no mês de referência",
    "V403411": "Número da faixa do rendimento/retirada em dinheiro",
    "V403412": "Qual foi o rendimento bruto/retirada que ... recebeu/fez nesse trabalho, no mês de referência ? (valor em dinheiro)",
    "V40342": "Recebeu/fez nesse trabalho no mês de referencia, rendimento/retirada em produtos e mercadorias",
    "V403421": "Número da faixa do rendimento/retirada em produtos e mercadorias",
    "V403422": "Qual foi o rendimento bruto/retirada que ... recebeu/fez nesse trabalho, no mês de referência ? (valor estimado dos produtos ou mercadorias)",
    "V4039": "Quantas horas ... trabalhava normalmente, por semana, nesse trabalho principal?",
    "V4039C": "Quantas horas ... trabalhou efetivamente na semana de referência nesse trabalho principal?",
    "V4040": "Até o dia ... (último dia da semana de referência) fazia quanto tempo que ... estava nesse trabalho ?",
    "V40401": "Fazia quanto tempo que estava nesse trabalho (de 1 mês a menos de 1 ano)",
    "V40402": "Fazia quanto tempo que estava nesse trabalho (de 1 ano a menos de 2 anos)",
    "V40403": "Fazia quanto tempo estava nesse trabalho (2 anos ou mais)",
    "V4041": "Código da ocupação (cargo ou função)",
    "V4043": "Nesse trabalho secundário, ... era",
    "V40431": "Trabalhador familiar não remunerado",
    "V4044": "Código da principal atividade desse negócio/empresa",
    "V4045": "Esse trabalho era na área",
    "V4046": "Esse negócio/empresa era registrado no Cadastro Nacional da Pessoa Jurídica - CNPJ?",
    "V4047": "Nesse trabalho secundário era funcionário público estatutário (federal, estadual ou municipal)?",
    "V4048": "Nesse trabalho secundário tinha carteira de trabalho assinada?",
    "V4049": "Era contribuinte de instituto de previdência por esse trabalho secundário?",
    "V4050": "Qual era o rendimento bruto mensal que ... recebia/fazia normalmente nesse trabalho secundário? (variável auxiliar)",
    "V40501": "Recebia/fazia normalmente nesse trabalho secundário rendimento/retirada em dinheiro",
    "V405011": "Número da faixa do rendimento/retirada em dinheiro",
    "V405012": "Valor em dinheiro do rendimento mensal que recebia normalmente nesse trabalho secundário",
    "V40502": "Recebia/fazia normalmente nesse trabalho secundário rendimento/retirada em produtos e mercadorias",
    "V405021": "Número da faixa do rendimento/retirada em produtos e mercadorias",
    "V405022": "Valor estimado dos produtos e mercadorias que recebia normalmente nesse trabalho secundário",
    "V40503": "Recebia/fazia normalmente nesse trabalho secundário rendimento/retirada somente em benefícios",
    "V405031": "Tipo de rendimento/retirada em benefícios que recebia/fazia nesse trabalho secundário",
    "V4051": "Qual foi o rendimento bruto que ... recebeu/fez nesse trabalho secundário, no mês de referência? (variável auxiliar)",
    "V40511": "Recebeu/fez nesse trabalho secundário rendimento/retirada em dinheiro no mês de referência",
    "V405111": "Número da faixa do rendimento/retirada em dinheiro",
    "V405112": "Valor em dinheiro do rendimento mensal que recebeu nesse trabalho secundário no mês de referência",
    "V40512": "Recebeu/fez nesse trabalho secundário no mês de referencia, rendimento/retirada em produtos e mercadorias",
    "V405121": "Número da faixa do rendimento/retirada em produtos e mercadorias",
    "V405122": "Valor estimado dos produtos e mercadorias que recebia recebeu nesse trabalho secundário no mês de referência",
    "V4056": "Quantas horas ... trabalhava normalmente, por semana, nesse trabalho secundário?",
    "V4056C": "Quantas horas ... trabalhou efetivamente na semana de referência nesse trabalho secundário?",
    "V4057": "Era contribuinte de instituto de previdência por esse(s) outro(s) trabalho(s)?",
    "V4058": "Qual era o rendimento bruto mensal que ... recebia/fazia normalmente nesse(s) outro(s) trabalho(s)? (variável auxiliar)",
    "V40581": "Recebia/fazia normalmente nesse(s) outro(s) trabalho(s) rendimento/retirada em dinheiro",
    "V405811": "Número da faixa do rendimento/retirada em dinheiro",
    "V405812": "Valor em dinheiro do rendimento mensal que recebia normalmente nesse(s) outro(s) trabalho(s)",
    "V40582": "Recebia/fazia normalmente nesse(s) outro(s) trabalho(s) rendimento/retirada em produtos e mercadorias",
    "V405821": "Número da faixa do rendimento/retirada em produtos e mercadorias",
    "V405822": "Valor estimado do produtos e mercadorias que recebia normalmente nesse(s) outro(s) trabalho(s)",
    "V40583": "Recebia/fazia normalmente nesse(s) outro(s) trabalho(s) rendimento/retirada somente em benefícios",
    "V405831": "Tipo de rendimento/retirada em benefícios que recebia/fazia nesse(s) outro(s) trabalho(s)",
    "V40584": "Não remunerado nesse(s) outro(s) trabalho(s)",
    "V4059": "Qual foi o rendimento bruto que ... recebeu/fez nesse(s) outro(s) trabalho(s), no mês de referência? (variável auxiliar)",
    "V40591": "Recebeu/fez nesse(s) outro(s) trabalho(s) rendimento/retirada em dinheiro no mês de referência",
    "V405911": "Número da faixa do rendimento/retirada em dinheiro",
    "V405912": "Valor em dinheiro do rendimento mensal que recebeu nesse(s) outro(s) trabalho(s) no mês de referência",
    "V40592": "Recebeu/fez nesse(s) outro(s) trabalhos no mês de referencia, rendimento/retirada em produtos e mercadorias",
    "V405921": "Número da faixa do rendimento/retirada em produtos e mercadorias",
    "V405922": "Valor estimado do produtos e mercadorias que recebia normalmente nesse(s) outro(s) trabalho(s) no mês de referência",
    "V4062": "Quantas horas ... trabalhava normalmente, por semana, nesse(s) outro(s) trabalho(s)?",
    "V4062C": "Quantas horas ... trabalhou efetivamente na semana de referência nesses outros trabalhos?",
    "V4063": "... gostaria de trabalhar mais horas do que as ... (soma das horas declaradas nas variáveis VD4031, VD4032, VD4033) horas que efetivamente trabalhou no(s) trabalho(s) que tinha na semana de referência?",
    "V4063A": "... gostaria de trabalhar mais horas do que as ... (soma das horas declaradas nos quesitos 39, 56 e 62) horas que normalmente trabalhava no(s) trabalho(s) que tinha na semana de referência?",
    "V4064": "No mês de referência, ... estaria disponível para trabalhar mais do que as (soma das horas declaradas nas variáveis VD4031, VD4032, VD4033) horas que efetivamente trabalhou no(s) trabalho(s) que tinha na semana de referência?",
    "V4064A": "No mês de referência, ... estaria disponível para trabalhar mais do que as (soma das horas declaradas nos quesitos 39, 56 e 62) horas que normalmente trabalhava no(s) trabalho(s) que tinha na semana de referência?",
    "V4071": "No período de ... a ... (período de referência de 30 dias), ... Tomou alguma providência para conseguir trabalho, seja um emprego ou um negócio próprio?",
    "V4072": "No período de ... a ... (período de referência de 30 dias), qual foi a principal providência que ... tomou para conseguir trabalho?",
    "V4072A": "No período de ... a ... (período de referência de 30 dias), qual foi a principal providência que ... tomou para conseguir trabalho?",
    "V4073": "Embora não tenha tomado providência para conseguir trabalho, gostaria de ter trabalhado na semana de ... a ... (semana de referência) ?",
    "V4074": "Qual foi o principal motivo de ... não ter tomado providência para conseguir trabalho no período de ... a ... (período de referência de 30 dias)?",
    "V4074A": "Qual foi o principal motivo de ... não ter tomado providência para conseguir trabalho no período de ... a ... (período de referência de 30 dias)?",
    "V4075A": "Quanto tempo depois de ... (último dia da semana de referência) irá começar esse trabalho que conseguiu?",
    "V4075A1": "Número de meses para começar o trabalho que conseguiu",
    "V4076": "Até o dia ... (último dia da semana de referência), fazia quanto tempo que ... estava sem qualquer trabalho e tentando conseguir trabalho?",
    "V40761": "fazia quanto tempo que vinha procurando trabalho (de 1 mês a menos de 1 ano)",
    "V40762": "fazia quanto tempo que vinha procurando trabalho (de 1 ano a menos de 2 anos)",
    "V40763": "fazia quanto tempo vinha procurando trabalho (2 anos ou mais)",
    "V4077": "Se tivesse conseguido um trabalho, ... poderia ter começado a trabalhar na semana de ... a ... (semana de referência)?",
    "V4078": "Qual foi o principal motivo para ... não querer(poder começar a) trabalhar na semana de ... a ... (semana de referência)?",
    "V4078A": "Qual foi o principal motivo para ... não querer(poder começar a) trabalhar na semana de ... a ... (semana de referência)?",
    "V4082": "No período de captação de 358 dias, ... trabalhou por pelo menos 1 hora?",
    "VD2002": "Condição no domicílio",
    "VD2003": "Número de componentes do domicílio (exclusive as pessoas cuja condição no domicílio era pensionista, empregado doméstico ou parente do empregado doméstico)",
    "VD2004": "Espécie da unidade doméstica",
    "VD2006": "Faixas etárias utilizadas no processo de calibração",
    "VD3004": "Nível de instrução mais elevado alcançado (pessoas de 5 anos ou mais de idade) padronizado para o Ensino fundamental - SISTEMA DE 9 ANOS",
    "VD3005": "Anos de estudo (pessoas de 5 anos ou mais de idade) padronizado para o Ensino fundamental - SISTEMA DE 9 ANOS",
    "VD3006": "Grupos de anos de estudo (pessoas de 5 anos ou mais de idade) padronizado para o Ensino fundamental - SISTEMA DE 9 ANOS",
    "VD4001": "Condição em relação à força de trabalho na semana de referência para pessoas de 14 anos ou mais de idade",
    "VD4002": "Condição de ocupação na semana de referência para pessoas de 14 anos ou mais de idade",
    "VD4003": "Força de trabalho potencial para pessoas de 14 anos ou mais de idade",
    "VD4004": "Subocupação por insuficiência de horas efetivamente trabalhadas em todos os trabalhos",
    "VD4004A": "Subocupação por insuficiência de horas habitualmente trabalhadas em todos os trabalhos",
    "VD4005": "Pessoas desalentadas na semana de referência",
    "VD4007": "Posição na ocupação no trabalho principal da semana de referência para pessoas de 14 anos ou mais de idade",
    "VD4008": "Posição na ocupação no trabalho principal da semana de referência para pessoas de 14 anos ou mais de idade (com subcategorias de empregados)",
    "VD4009": "Posição na ocupação e categoria do emprego do trabalho principal da semana de referência para pessoas de 14 anos ou mais de idade",
    "VD4010": "Grupamentos de atividade principal do empreendimento do trabalho principal da semana de referência para pessoas de 14 anos ou mais de idade",
    "VD4011": "Grupamentos ocupacionais do trabalho principal da semana de referência para pessoas de 14 anos ou mais de idade",
    "VD4012": "Contribuição para instituto de previdência em qualquer trabalho da semana de referência para pessoas de 14 anos ou mais de idade",
    "VD4013": "Faixa das horas habitualmente trabalhadas por semana em todos os trabalhos para pessoas de 14 anos ou mais de idade",
    "VD4014": "Faixa das horas efetivamente trabalhadas na semana de referência em todos os trabalhos para pessoas de 14 anos ou mais de idade",
    "VD4015": "Tipo de remuneração habitualmente recebida no trabalho principal para pessoas de 14 anos ou mais de idade",
    "VD4016": "Rendimento mensal habitual do trabalho principal para pessoas de 14 anos ou mais de idade (apenas para pessoas que receberam em dinheiro, produtos ou mercadorias no trabalho principal)",
    "VD4017": "Rendimento mensal efetivo do trabalho principal para pessoas de 14 anos ou mais de idade (apenas para pessoas que receberam em dinheiro, produtos ou mercadorias no trabalho principal)",
    "VD4018": "Tipo de remuneração habitualmente recebida em todos os trabalhos para pessoas de 14 anos ou mais de idade",
    "VD4019": "Rendimento mensal habitual de todos os trabalhos para pessoas de 14 anos ou mais de idade (apenas para pessoas que receberam em dinheiro, produtos ou mercadorias em qualquer trabalho)",
    "VD4020": "Rendimento mensal efetivo de todos os trabalhos para pessoas de 14 anos ou mais de idade (apenas para pessoas que receberam em dinheiro, produtos ou mercadorias em qualquer trabalho)",
    "VD4023": "Motivo pelo qual não procurou trabalho ou não gostaria de ter trabalhado ou não estava disponível para iniciar um trabalho",
    "VD4030": "Motivo pelo qual não procurou trabalho ou não gostaria de ter trabalhado ou não estava disponível para iniciar um trabalho",
    "VD4031": "Horas habitualmente trabalhadas por semana em todos os trabalhos para pessoas de 14 anos ou mais de idade",
    "VD4032": "Horas efetivamente trabalhadas na semana de referência no trabalho principal para pessoas de 14 anos ou mais de idade",
    "VD4033": "Horas efetivamente trabalhadas na semana de referência no trabalho secundário para pessoas de 14 anos ou mais de idade",
    "VD4034": "Horas efetivamente trabalhadas na semana de referência nos outros trabalhos para pessoas de 14 anos ou mais de idade",
    "VD4035": "Horas efetivamente trabalhadas na semana de referência em todos os trabalhos para pessoas de 14 anos ou mais de idade",
    "VD4036": "Faixa das horas habitualmente trabalhadas por semana no trabalho principal para pessoas de 14 anos ou mais de idade",
    "VD4037": "Faixa das horas efetivamente trabalhadas na semana de referência no trabalho principal para pessoas de 14 anos ou mais de idade"
}

ALL_GROUP = [
    "Ano", "Trimestre", "UF", "V1028", "Capital", "RM_RIDE", "V1022", "V1023",
    "V2001", "V2005", "V2007", "V2008", "V20081", "V20082", "V2009", "V2010",
    "V3001", "V3002", "V3002A", "V3003", "V3003A", "V3004", "V3005", "V3005A",
    "V3006", "V3006A", "V3007", "V3008", "V3009", "V3009A", "V3010", "V3011",
    "V3011A", "V3012", "V3013", "V3013A", "V3013B", "V3014", "V4001", "V4002",
    "V4003", "V4004", "V4005", "V4006", "V4006A", "V4007", "V4008", "V40081",
    "V40082", "V40083", "V4009", "V4010", "V4012", "V40121", "V4013", "V40132",
    "V40132A", "V4014", "V4015", "V40151", "V401511", "V401512", "V4016",
    "V40161", "V40162", "V40163", "V4017", "V40171", "V401711", "V4018",
    "V40181", "V40182", "V40183", "V4019", "V4020", "V4021", "V4022", "V4024",
    "V4025", "V4026", "V4027", "V4028", "V4029", "V4032", "V4033", "V40331",
    "V403311", "V403312", "V40332", "V403321", "V403322", "V40333", "V403331",
    "V4034", "V40341", "V403411", "V403412", "V40342", "V403421", "V403422",
    "V4039", "V4039C", "V4040", "V40401", "V40402", "V40403", "V4041", "V4043",
    "V40431", "V4044", "V4045", "V4046", "V4047", "V4048", "V4049", "V4050",
    "V40501", "V405011", "V405012", "V40502", "V405021", "V405022", "V40503",
    "V405031", "V4051", "V40511", "V405111", "V405112", "V40512", "V405121",
    "V405122", "V4056", "V4056C", "V4057", "V4058", "V40581", "V405811",
    "V405812", "V40582", "V405821", "V405822", "V40583", "V405831", "V40584",
    "V4059", "V40591", "V405911", "V405912", "V40592", "V405921", "V405922",
    "V4062", "V4062C", "V4063", "V4063A", "V4064", "V4064A", "V4071", "V4072",
    "V4072A", "V4073", "V4074", "V4074A", "V4075A", "V4075A1", "V4076",
    "V40761", "V40762", "V40763", "V4077", "V4078", "V4078A", "V4082",
    "VD2002", "VD2003", "VD2004", "VD2006", "VD3004", "VD3005", "VD3006",
    "VD4001", "VD4002", "VD4003", "VD4004", "VD4004A", "VD4005", "VD4007",
    "VD4008", "VD4009", "VD4010", "VD4011", "VD4012", "VD4013", "VD4014",
    "VD4015", "VD4016", "VD4017", "VD4018", "VD4019", "VD4020", "VD4023",
    "VD4030", "VD4031", "VD4032", "VD4033", "VD4034", "VD4035", "VD4036",
    "VD4037"
]

MAIN_GROUP = [
    "Ano", "Trimestre", "UF", "V1028", "VD2002", "VD2003", "VD2004", "VD2006",
    "VD3004", "VD3005", "VD3006", "VD4001", "VD4002", "VD4003", "VD4004",
    "VD4004A", "VD4005", "VD4007", "VD4008", "VD4009", "VD4010", "VD4011",
    "VD4012", "VD4013", "VD4014", "VD4015", "VD4016", "VD4017", "VD4018",
    "VD4019", "VD4020", "VD4023", "VD4030", "VD4031", "VD4032", "VD4033",
    "VD4034", "VD4035", "VD4036", "VD4037"
]

WORK_GROUP = [
    "Ano", "Trimestre", "UF", "V1028", "VD4001", "VD4002", "VD4003", "VD4002",
    "VD4004A", "VD4005", "VD4016", "VD4017", "VD4019", "VD4020", "VD4007",
    "VD4008", "VD4009", "VD4010", "VD4011", "VD4012", "VD4013", "VD4014",
    "VD4015", "VD4018", "VD4023", "VD4030", "VD4031", "VD4032", "VD4033",
    "VD4034", "VD4035", "VD4036", "VD4037"
]

EDUCATION_GROUP = [
    "Ano", "Trimestre", "UF", "V1028", "VD3004", "VD3005", "VD3004", "VD3006"
]

NUMERICAL_GROUP = [
    "Ano", "Trimestre", "V2001", "V2008", "V20081", "V20082", "V2009",
    "V40081", "V40082", "v40083", "V401511", "V401512", "V40161", "V40162",
    "V40163", "V401711", "V40181", "V40182", "V40183", "V403312", "V403322",
    "V403412", "V403422", "V4039", "V4039C", "V40401", "V40402", "V40403",
    "V405012", "V405022", "V405112", "V405122", "V4056", "V4056C", "V405812",
    "V405822", "V405912", "V405922", "V4062", "V4062C", "V4075A1", "V40761",
    "V40762", "V40763", "VD2003", "VD4016", "VD4017", "VD4019", "VD4020",
    "VD4031", "VD4032", "VD4033", "VD4034", "VD4035"
]

DEFLATOR_GROUP = [
    "VD4016", "VD4017", "VD4019", "VD4020"
]

REGULAR_INCOME_GROUP = [
    "VD4016", "VD4019"
]



import io
import zipfile
import requests
import textwrap
import pyarrow as pa
import pyarrow.parquet as pq
import polars as pl


def download_zip_content_from_url(url, folder):
    response = requests.get(url)
    if response.status_code != 200:
        return False
    zip = zipfile.ZipFile(io.BytesIO(response.content))
    zip.extractall(folder)
    return True


def save_parquet_from_rows(rows, col_names, parquet_writer, dtypes):
    df = pl.DataFrame(rows, orient='row')
    df.columns = col_names
    df = df.cast(dtypes, strict=False)
    parquet_writer.write_table(df.to_arrow())


def convert_fwf_to_parquet(fwf_file, parquet_file, columns, schema):
    names = list(columns.keys())
    widths = list(columns.values())
    chunk_size = 50_000
    parquet_writer = pq.ParquetWriter(parquet_file, pa.schema(schema))
    dtypes = {col: getattr(pl, dtype) for col, dtype in schema.items()}
    with open(fwf_file, 'r') as f_in:
        rows = []
        for line in f_in:
            row = [line[start:end].strip() for start, end in widths]
            rows.append(row)
            if len(rows) >= chunk_size:
                save_parquet_from_rows(rows, names, parquet_writer, dtypes)
                rows = []
        if len(rows) > 0:
            save_parquet_from_rows(rows, names, parquet_writer, dtypes)
    parquet_writer.close()


def get_vars_from_bin_dict(bin_dict):
    return [k for k, v in bin_dict.items() if v == 1]


def wrap_string(text, max_length):
    return textwrap.fill(text, width=max_length)


def get_var_desc(vars_desc, var, show_var=True):
    if show_var:
        return f"[{var}] {vars_desc[var]}"
    return vars_desc[var]


def has_duplicates(list_):
    aux = []
    for e in list_:
        if e not in aux:
            aux.append(e)
    return len(list_) != len(aux)


def is_numeric(value):
    try:
        float(value)
        return True
    except:
        return False
